//*****************************************************************************
// das_dig.c module contains routines to process messages sent by a client to
// preform the function of the 'das' driver routines.  The operation of these
// functions will call the low level 'das' driver to preform the actual 
// operation. 
//*****************************************************************************
// The following is the list of functions in the module. 
//
//     Message processing function     DAS driver function
//     ---------------------------     -------------------
//       das_dig_in_msg             -     das_digital_in
//       das_dig_out_msg            -     das_digital_out
//*****************************************************************************
#include <errno.h>

#include <das.h>
#include "struct.h"
#include "proto.h"
#include "extern.h"

//*****************************************************************************
// das_dig_in_msg -  Read a digital port form the adapter.
//*****************************************************************************
int das_dig_in_msg(pid_t pid)
{
   struct ocb *ocb;

                     // Get 'ocb' assoicated with calling process.      
   if((ocb = get_ocb(pid, io_msg.das_din.fd)) == -1)
   {
      io_msg.status = EBADF;
      return(REPLY);
   }
                     // Check access mode & access flags.
   if ((ocb->mode & S_IREAD) == 0)
   {
      io_msg.status = EACCES;
      return(REPLY);
   }
                     // Read port.
   if( das_digital_in(das_fd, io_msg.das_din.port, 
                        &io_msg.das_din_reply.bits) == -1)
      io_msg.status = errno;
   else
      io_msg.status = EOK;
   return(REPLY);
}

//*****************************************************************************
// das_dig_out_msg - Write to a digital port.
//*****************************************************************************
int das_dig_out_msg(pid_t pid)
{
   struct ocb *ocb;

                     // Get 'ocb' assoicated with calling process.   
   if((ocb = get_ocb(pid, io_msg.das_dout.fd)) == -1)
   {
      io_msg.status = EBADF;
      return(REPLY);
   }
                     // Check access mode & access flags.
   if ((ocb->mode & S_IWRITE) == 0)
   {
      io_msg.status = EACCES;
      return(REPLY);
   }
                     // Write to port.
   if (das_digital_out(das_fd, io_msg.das_dout.port, io_msg.das_dout.bits,
                        io_msg.das_dout.mask, &io_msg.das_dout_reply.old_bits,
                        &io_msg.das_dout_reply.new_bits) == -1)
      io_msg.status = errno;
   else
      io_msg.status = EOK;
   return(REPLY);
}
