#ifdef __USAGE

%C [-b <nbufs> -d -f <file> -P <prefix> -S <size> -s <sample rate>] &
   
  -b nbufs        Allocate this many buffers (default 10).
  -d              Allocate buffers as "DMA-safe".
  -f file         Use this configuration file. (default is /usr/lib/das/config
                                                /das_default.xxx)
  -P prefix       Prefix to register under /dev (defualt das).
  -S size         Set the buffers to this size (default is 1000 bytes).
  -s sample rate  Sample rate per channel maximum 10000 (defualt 10000).

Example:

%C -b 5 -P pressure -S 500 -s 5000 &

#endif             

#include <errno.h>
#include <unistd.h>
#include <sys/kernel.h>

#include <das.h>
#include "struct.h"
#include "proto.h"

union msg               io_msg;
struct stat             das_stat;
struct ocb              *excl_ocb;
time_t far              *timep;
void                    *fd_ctrl;
int                     owner_pid = 0;
int                     das_fd;
int                     num_ai;
int                     in_use = NO_USERS;

int                     tickque;
struct qent             q[MAX_TQUE];
volatile struct qtick   qticks;
long                    sample_r;

int                     gbuf_size;
int                     gnum_bufs;
int                     gdma_flag;
struct das_mem_que      *glb_free_list;

pid_t                   hw_adproxy, hw_daproxy;
das_info                d_info;
das_buf_t               *das_mem_bufs;
das_buf_t               *das_ad_data;

void main(int argc, char *argv[])
{
   pid_t pid;
   int reply;

   das_init(argc, argv);                     // Initialize acquisition manager.

   close(STDIN_FILENO);
   close(STDOUT_FILENO);
   close(STDERR_FILENO);

   for(;;)                                   // Message processing loop.
   {
      pid = Receive(0, &io_msg, sizeof(io_msg));
      
                     // Check to see if it is the proxy from the interrupt 
                     // handler. 
      if (pid == hw_adproxy && in_use != SINGLE_USER)
      {
         das_get_conversion();
         continue;
      }
      else if(in_use == SINGLE_USER)
      {
         if (pid ==  hw_adproxy)
         {
            excl_ad_proxy_msg();
            continue;
         }
         if (pid ==  hw_daproxy)
         {
            excl_da_proxy_msg();
            continue;
         }
      }
                     // After receiving message, call function to process it 
                     // if supported - reply with ENOSYS if not supported.   
      switch(io_msg.type)
      {
         case _SYSMSG:
            reply = sys_msg(pid);
            break;

         case _IO_HANDLE:
            reply = handle_msg(pid);
            break;

         case _IO_OPEN:                    
            reply = open_msg(pid);
            break;

         case _IO_CLOSE:
            reply = close_msg(pid);
            break;

         case _IO_FSTAT:
            reply = fstat_msg(pid);
            break;

         case _IO_STAT:
            reply = stat_msg(pid);
            break;

         case _IO_CHMOD:
            reply = chmod_msg(pid);
            break;

         case _IO_CHOWN:
            reply = chown_msg(pid);
            break;

         case _IO_UTIME:
            reply = utime_msg(pid);
            break;

         case _IO_DAS:
            switch(io_msg.das_msg.request)
            {
               case _DAS_QUERY:
                  reply = das_query_msg(pid);
                  break;

               case _DAS_ALLOC:
                  reply = das_alloc_msg(pid);
                  break;                  
               
               case _DAS_FREE:
                  reply = das_free_msg(pid);
                  break;

               case _DAS_AD_SYNC:
                  reply = das_ad_sync_msg(pid);
                  break;
                  
               case _DAS_AD_ENQUE:
                  reply = das_ad_enqueue_msg(pid);
                  break;
               
               case _DAS_AD_READ:
               case _DAS_AD_CREAD:
                  reply = das_ad_read_msg(pid);
                  break;

               case _DAS_AD_TERM:
                  reply = das_ad_term_msg(pid);
                  break;
                  
               case _DAS_AD_SET_SMP:
                  reply = das_ad_sam_msg(pid);
                  break;
                  
               case _DAS_AD_SET_SCAN:
                  reply = das_ad_set_scan_msg(pid);
                  break;
                  
               case _DAS_AD_GET_RANGE:
                  reply = das_ad_get_rng_msg(pid);
                  break;
                              
               case _DAS_AD_SET_RANGE:
                  reply = das_ad_set_rng_msg(pid);
                  break;

               case _DAS_AD_GET_TRIG:
                  reply = das_ad_get_trig_msg(pid);
                  break;

               case _DAS_AD_SET_TRIG:
                  reply = das_ad_set_trig_msg(pid);
                  break;

               case _DAS_AD_GET_RDY_QUE:
               case _DAS_AD_GET_FIN_QUE:
                  reply = das_ad_get_que_msg(pid);
                  break;
               
               case _DAS_DA_SYNC:
                  reply = das_da_sync_msg(pid);
                  break;
                  
               case _DAS_DA_ENQUE:
                  reply = das_da_enqueue_msg(pid);
                  break;
               
               case _DAS_DA_WAIT:
               case _DAS_DA_CWAIT:
                  reply = das_da_wait_msg(pid);
                  break;
                  
               case _DAS_DA_TERM:
                  reply = das_da_term_msg(pid);
                  break;

               case _DAS_DA_SET_SMP:
                  reply = das_da_sam_msg(pid);
                  break;
                 
               case _DAS_DA_GET_RANGE:
                  reply = das_da_get_rng_msg(pid);
                  break;
                  
               case _DAS_DA_SET_RANGE:
                  reply = das_da_set_rng_msg(pid);
                  break;

               case _DAS_DA_SET_TRIG:
                  reply = das_da_set_trig_msg(pid);
                  break;
               
               case _DAS_DA_GET_TRIG:
                  reply = das_da_get_trig_msg(pid);
                  break;
                  
               case _DAS_DA_GET_RDY_QUE:
               case _DAS_DA_GET_FIN_QUE:
                  reply = das_da_get_que_msg(pid);
                  break;

               case _DAS_DIG_IN:
                  reply = das_dig_in_msg(pid);
                  break;
                  
               case _DAS_DIG_OUT:
                  reply = das_dig_out_msg(pid);
                  break;
                  
               default:
                  io_msg.status = ENOSYS;
                  reply = REPLY;
                  break;
            }
            break;

         // The following messages are not supported by this driver. 
         case _IO_READ:      
         case _IO_WRITE:
         case _IO_QIOCTL:
         case _IO_LSEEK:         
         case _IO_RENAME:
         case _IO_GET_CONFIG:
         case _IO_FLAGS:
         case _IO_LOCK:
         case _IO_READDIR:
         case _IO_REWINDDIR:
         case _IO_CHDIR:
         case _IO_DUP:
         default:
            io_msg.status = ENOSYS;
            reply = REPLY;
            break;
      }
      if(reply)
         Reply(pid, &io_msg, sizeof(io_msg));
   }
}
