//*****************************************************************************
// das_mem_r.c - This module conatins routines to respond to das_alloc and
// das_free messages from a client.  Also 2 support routines are included which
// are used by the acquistions manager to obatain pre-allocated buffers for 
// a particular request.
//*****************************************************************************
// The following is the list of the functions in this module.
//
//
//     Message processing function     DAS driver function 
//     ---------------------------     -------------------
//         das_alloc_msg            -     das_alloc
//         das_free_msg             -     das_free
//
//         das_get_buf              -     used to obtain data buffer.
//         das_rel_buf              -     used to release data buffer.
//*****************************************************************************

#include "proto.h"
#include "extern.h"
#include "struct.h"
#include "define.h"

//*****************************************************************************
// das_alloc_msg() - Try and give the client the requested number of buffers
//                   if they are big enough.
//*****************************************************************************
int das_alloc_msg(pid_t pid)
{
   struct ocb *ocb;
   struct das_mem_que *que_ptr;
   int i;
   
                     // Get 'ocb' assoicated with calling process.      
   if ((ocb = get_ocb(pid, io_msg.das_alloc.fd)) == -1)
   {
      io_msg.status = EBADF;
      return(REPLY);
   }
   io_msg.status = EOK;
   if (in_use == SINGLE_USER)
   {
      if (io_msg.das_alloc.flags != gdma_flag)
      {
         io_msg.status = ENOMEM;
         return(REPLY);
      }
   }                 // Are there enough buffers and are they big enough.
   if ((io_msg.das_alloc.n_bufs > gnum_bufs) || 
         (io_msg.das_alloc.n_bytes > gbuf_size))
      io_msg.status = ENOMEM;
   else              // Move a list buffers to clients OCB.
   {
      que_ptr = glb_free_list;
      ocb->free_list = que_ptr;
      for (i = 0; i < io_msg.das_alloc.n_bufs-1; i++)
         que_ptr = que_ptr->n_que;
      glb_free_list = que_ptr->n_que;
      que_ptr->n_que = NULL;
      gnum_bufs -= io_msg.das_alloc.n_bufs;
      ocb->num_bufs = io_msg.das_alloc.n_bufs;
   }
   return(REPLY);
}

//*****************************************************************************
// das_free_msg() -  Return any buffers given to the client.
//*****************************************************************************
int das_free_msg(pid_t pid)
{
   struct ocb *ocb;
   struct das_mem_que *que_ptr;
   
                     // Get 'ocb' assoicated with calling process.      
   if ((ocb = get_ocb(pid, io_msg.das_alloc.fd)) == -1)
   {
      io_msg.status = EBADF;
      return(REPLY);
   }
   io_msg.status = EOK;

   if (ocb->free_list !=NULL)                // Move the list of buffers back to 
   {                                         // the global free list.
      gnum_bufs += ocb->num_bufs;         
      que_ptr = glb_free_list;
      if (que_ptr == NULL)
         glb_free_list = ocb->free_list;
      else
      {
         while (que_ptr->n_que)
            que_ptr = que_ptr->n_que;
         que_ptr->n_que = ocb->free_list;
      }
      ocb->free_list = NULL;
   }
   return(REPLY);
}

//*****************************************************************************
// das_get_buf() -   Get a free buffer from the clients OCB.
//*****************************************************************************
struct das_mem_que * das_get_buf(struct ocb *ocb)
{
   struct das_mem_que *que_ptr;
   
   que_ptr = ocb->free_list;
   if (que_ptr != NULL)
   {
      ocb->free_list = ocb->free_list->n_que;
      que_ptr->n_que = NULL;
   }
   return(que_ptr);
}

//*****************************************************************************
// das_rel_buf -  Release a buffer back to the clients OCB. 
//*****************************************************************************
void das_rel_buf(struct das_mem_que * que, struct ocb *ocb)
{
   struct das_mem_que *que_ptr;
   
   que_ptr = ocb->free_list;
   if (que_ptr == NULL)
      ocb->free_list = que;
   else
   {
      while(que_ptr->n_que)
         que_ptr = que_ptr->n_que;
      que_ptr->n_que = que;
   }
}
