//***************************************************************************
// das_queue.c module contains routines to:
//***************************************************************************
//    allocate a queue entry           - das_res_create_entry()
//    unlink an entry from the queue   - das_res_unlink()
//    link an entry to a queue         - das_res_link()
//***************************************************************************
#include <stdlib.h>
#include <errno.h>
#include "struct.h"

//***************************************************************************
// das_res_queue -   Allocates a entry for the queue and returns a pointer to
//                   it.
//***************************************************************************
struct das_res_queue * das_res_create_entry(void)
{
   struct das_res_queue *ptr;

   if((ptr = (struct das_res_queue *)calloc(1, sizeof(struct das_res_queue)))
                   == NULL)
      errno = ENOMEM;
   return(ptr);
}

//*****************************************************************************
// das_res_unlink -  Unlinks the entry from the queue and free it's memory if 
//                   'op' is FREE.  
//*****************************************************************************
void das_res_unlink(int op, struct das_res_queue **hptr, 
                  struct das_res_queue **tptr)
{
   struct   das_res_queue  *tmp;
   
   tmp = *hptr;
   *hptr =(*hptr)->nptr;
   if(!*hptr)
      *tptr = NULL;
   if(op)                           // free the memory
      free(tmp);
}

//*****************************************************************************
// das_res_link - Links an entry to the tail of the queue.  It returns true if 
//                it is the first entry in the queue.  
//*****************************************************************************
int das_res_link(struct das_res_queue *ptr, struct das_res_queue **hptr, 
            struct das_res_queue **tptr)
{
   // If there is already entries in the queue put it at the tail and return
   if(*tptr)
   {
      (*tptr)->nptr = ptr;
      *tptr = ptr;
   }
   else              // Nothing in the queue, initiate conversions
   {
      *hptr = *tptr = ptr;
      return(1);
   }
   return(EOK);
}
