//*****************************************************************************
// das_support.c module contains the support routines used by the acquistion
// manager.
//*****************************************************************************
// The following is the list of functions in this module.
//
//       -  map_ocb
//       -  get_ocb
//       -  check_access
//       -  get_ids
//*****************************************************************************
#include <string.h>
#include <fcntl.h>
#include <sys/psinfo.h>
#include <sys/fd.h>
#include "struct.h"
#include "extern.h"
#include "proto.h"

//*****************************************************************************
// map_ocb -
//*****************************************************************************
int map_ocb(int pid, int fd, struct ocb *ocb)
{
                     // Use unit as index into ocb array. 
   if(qnx_fd_attach(pid, fd, 0, 0, 0, 0, (unsigned)ocb) == -1)
      return(-1);
   return(0);
}

//*****************************************************************************
// get_ocb -
//*****************************************************************************
struct ocb * get_ocb( pid_t pid, int fd)
{
   struct ocb *ocb;
   
   ocb = (struct ocb *)__get_fd(pid, fd, fd_ctrl);
   if(ocb == (struct ocb *)0 || ocb == (struct ocb *)-1)
      return((struct ocb *)-1);
   return(ocb);
}

//*****************************************************************************
// check_access -
//*****************************************************************************
mode_t check_access(pid_t pid, struct stat *sptr)
{
   mode_t mode;
   uid_t uid;
   gid_t gid;

   mode_t perm_mode[3] = { S_IREAD, S_IWRITE, S_IREAD|S_IWRITE };

                     // Get egid and euid of process pid.
   get_ids(pid, &uid, &gid);
                     // Make sure the calling process has correct permissions.
   mode = perm_mode[io_msg.open.oflag & O_ACCMODE];
   mode &=  (uid == 0) ?
               S_IRWXU : (uid == sptr->st_uid) ?   (sptr->st_mode & S_IRWXU) :
               (gid == sptr->st_gid) ? (sptr->st_mode & S_IRWXG) << 3 :
                  (sptr->st_mode & S_IRWXO) << 6;
   return(mode);
}

//*****************************************************************************
// get_ids -
//*****************************************************************************
void get_ids (pid_t pid, uid_t *uid, gid_t *gid)
{
   struct _psinfo3    psinfo3;
   
   __get_pid_info(pid, &psinfo3, fd_ctrl);
   *uid = psinfo3.euid;
   *gid = psinfo3.egid;
}

