//*****************************************************************************
// das_timer.c module contains support function used by the acquistion manager 
// to perform timed A/D and D/A operations.  
//*****************************************************************************
// The following is the list of functions in this module.
//
//       - das_add_tick_list
//       - das_rm_tick_list
//       - get_tick_list_id
//       - das_get_conversion
//       - das_res_int
//*****************************************************************************
#include <das_inline.h>
#include "define.h"
#include "struct.h"
#include "extern.h"
#include "proto.h"

//*****************************************************************************
// das_add_tick_list -  Add to tick list.
//*****************************************************************************
int das_add_tick_list(struct ocb *ocb, long ticks)
{
   int next, psw, t_id;

   if ((t_id = get_tick_list_id()) == -1)
      return(-1);
   q[t_id].qticks.small = ticks;
   psw = disable();
   if ((q[t_id].qticks.small += qticks.small) >= BILLION)
   {
      q[t_id].qticks.small -= BILLION;
      q[t_id].qticks.big++;
   }
   q[t_id].qticks.big += qticks.big;
   restore(psw);
   
   // Place in time queue.
   for ( next = get_next(tickque); next != tickque; next = get_next(next))
   {
      if (q[t_id].qticks.big < get_ticks(next).big || 
         ( q[t_id].qticks.big == get_ticks(next).big && 
         (q[t_id].qticks.small < get_ticks(next).small)))
         break;
   }
   q[t_id].qnext = next;
   q[t_id].qprev = q[next].qprev;
   q[q[next].qprev].qnext = t_id;
   q[next].qprev = t_id;
   q[t_id].ocb = ocb;
   return(t_id);
}

//*****************************************************************************
// das_rm_tick_list -   Remove from tick list.
//*****************************************************************************
struct ocb * das_rm_tick_list(int t_id)
{
   struct ocb *ocb;
   int psw;
   
   psw = disable();
   q[q[t_id].qprev].qnext = q[t_id].qnext;
   q[q[t_id].qnext].qprev = q[t_id].qprev;
   restore(psw);
   ocb = q[t_id].ocb;
   q[t_id].ocb = NULL;
   return(ocb);
}
   
//*****************************************************************************
// get_tick_list_id -   Get a free list entry. 
//*****************************************************************************
int get_tick_list_id(void)
{
   int i;
   
   for (i = tickque+1; i < MAX_TQUE; i++)
      if (q[i].ocb == NULL)
         return(i);
      
   return(-1);
}

//*****************************************************************************
// das_get_conversion - Perform D/A or A/D conversion for client.
//*****************************************************************************
void das_get_conversion()
{
   struct qtick cur_ticks;
   struct ocb *ocb;
   int psw, next;
   
   psw = disable();
   cur_ticks = qticks;
   restore(psw);
   
   for (next = get_next(tickque); next != tickque; next = get_next(next))
   {
      if (cur_ticks.big > get_ticks(next).big || 
         (cur_ticks.big == get_ticks(next).big && 
         (cur_ticks.small >= get_ticks(next).small)))
      {
         ocb = das_rm_tick_list(next);
         if (ocb->req_t == A_D)
            das_get_ad_sample(ocb);
         else if (ocb->req_t == D_A)
            das_put_da_sample(ocb);
      }
   }
} 

//*****************************************************************************
// das_res_int -  Called from interrupt routine within direct library to 
//                increament global tick count and determine weather a proxy
//                should be triggered. (See direct call interupt handler.
//*****************************************************************************
#pragma off (check_stack)
int das_res_int(void)
{
   int next;
   
   if(++qticks.small == BILLION)
   {
      qticks.small = 0;
      qticks.big++;              
   }
   if((next=get_next(tickque)) != tickque && ((q[next].qticks.big < 
      qticks.big) || (q[next].qticks.big == qticks.big) && 
         (q[next].qticks.small <= qticks.small)))
      return(TRUE);  
   return(FALSE);
}
#pragma on (check_stack)
