#include <sys/fd.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <das.h>
#include "struct.h"
#include "proto.h"
#include "extern.h"

//*****************************************************************************
// open_msg -  Perform checks on application opening driver, and setup its
//             Open Control Block.  
//*****************************************************************************
int open_msg(pid_t pid)
{
   mode_t mode;
   struct ocb  *ocb;
   int entry;
                     // Check permissions 
   if (((mode = check_access(pid, &das_stat)) & S_IRWXU) == 0)
   {
      io_msg.status = EPERM;
      return(REPLY);
   }
                     // Check for exclusive access to acquistion manager
   if (io_msg.open.oflag & O_EXCL && in_use == NO_USERS)
   {
      in_use = SINGLE_USER;
      das_ad_set_sample(das_fd, 0, d_info.pace_clk/1000);
   }
   else
   {
      if (in_use == SINGLE_USER)
      {
         io_msg.status = EBUSY;
         return(REPLY);
      }
      if (++in_use == 1)
      {
         das_reinit();
         das_start_dma();
      }
      else if (in_use > MAX_USERS)
      {
         in_use--;
         io_msg.status = EBUSY;
         return(REPLY);
      }
   }
      
   if ((ocb = (struct ocb *)calloc(1, sizeof(struct ocb))) == NULL)
   {
      io_msg.status = errno;
      return(REPLY);
   }
                     // Map PID & FD to an Open Control Block.
   if (map_ocb(pid, io_msg.open.fd, ocb))
   {
      io_msg.status = errno;
      return(REPLY);
   }
   ocb->pid = pid;
   ocb->count++;                    
   ocb->ad.ticks = 10;                       // Default A/D sampling rate
   ocb->da.ticks = 10;                       // Default D/A sampling rate.
   ocb->mode = mode;                         // Save access mode.
   ocb->req_t = NONE;
   if(in_use == SINGLE_USER)                 
      excl_ocb = ocb;                        // Setup global OCB.
   else
   {                                         // Defualt scan sequence.
      for (entry = 0; entry < NUM_ENT; entry++)
         ocb->ad_seq.seq[entry] = entry;
      ocb->ad_seq.n_entries = entry;
   }
   io_msg.status = EOK;
   return(REPLY);
}

//*****************************************************************************
// das_query_msg -   Return information about the hardware back to the 
//                   application.
//*****************************************************************************
#pragma off (unreferenced)
int das_query_msg(pid_t pid)
{
   das_info info;
   io_msg.das_query_reply.status = EOK;
                     // Return hardware info (port, irq, etc.)
   das_get_info(das_fd, &info);
   if (in_use != SINGLE_USER)
      info.pace_clk /= sample_r;
   memcpy(&io_msg.das_query_reply.d_info, &info, sizeof(das_info)); 
   return(REPLY);
}
#pragma on (unreferenced)
