#ifndef _DAS_STRUCT

#include <stdio.h>
#include <errno.h>
#include <sys/sys_msg.h>
#include <sys/io_msg.h>
#include <sys/stat.h>

#include <das.h>
#include "define.h"
#include "das_msg.h"

// das_t data type for D/A and A/D functions
#ifndef _DAS_T_DEFINED_

#ifdef __386__
   typedef  unsigned short int         das_t;
#else
   typedef  unsigned short int __far   das_t;   
#endif

#define  _DAS_T_DEFINED_
#endif

struct das_seq
{
   short int               n_entries;
   char                    seq[NUM_ENT];
};

struct das_mem_que
{
   struct das_mem_que      *n_que;
   das_buf_t               *buffer;
};

struct das_res_queue
{
   struct das_res_queue    *nptr;      
   int                     nconv;      // # of conversions to fo
   struct das_mem_que      *mem_que;         
   das_t                   *cp;        // Current pointer within buffer.
   int                     cur_conv;   // # of conversions left to do.
   int                     chnl;       // Current channel 
   int                     mode;       
   pid_t                   fin_proxy;  // Used to notify client.
   int                     que_req_t;  // Request type 
};

struct das_res_chnl                    // used for A/D and D/A
{
   long                    ticks;      // conversion rate in Hz.
   int                     n_rent;     // number of entries in ready queue
   int                     n_fent;     // number of entries in finish queue
   int                     reuse;      // reuse flag.
   struct   das_res_queue  *f_hd;      // head of finish queue
   struct   das_res_queue  *f_tl;      // tail of finish queue
   struct   das_res_queue  *r_hd;      // head of ready queue
   struct   das_res_queue  *r_tl;      // tail of ready queue
};
            
struct ocb
{
   pid_t                   pid;              
   int                     t_id;
   mode_t                  mode;             // Access modes.
   int                     count;            // Number of fd's on the ocb.
   int                     req_t;
   struct das_mem_que      *free_list;
   int                     num_bufs;
   struct das_seq          ad_seq;           // Scan array.
   int                     chnl_ent;         // Current channel/entry number 
   struct das_res_chnl     ad;
   struct das_res_chnl     da;
};

struct qtick
{
   unsigned long           small;
   unsigned long           big;
};

struct qent
{
   int                     qprev;
   int                     qnext;
   struct qtick            qticks;
   struct ocb              *ocb;
};

      /* Messages supported by driver (structures are in /sys/io_msg.h). */

union msg
{
   short unsigned                type;
   short unsigned                status;
   struct _sysmsg_hdr            sysmsg;
   struct _sysmsg_hdr_reply      sysmsg_reply;
   struct _io_open               open;          /* IO_HANDLE uses this one also. */
   struct _io_open_reply         open_reply;
   struct _io_dup                dup;           /* Used for fork, exec, dup, dup2()'s. */
   struct _io_dup_reply          dup_reply;
   struct _io_close              close;
   struct _io_close_reply        close_reply;
   struct _io_read               read;
   struct _io_read_reply         read_reply;
   struct _io_ioctl              ioctl;
   struct _io_ioctl_reply        ioctl_reply;
   struct _io_utime              utime;
   struct _io_utime_reply        utime_reply;
   struct _io_chmod              chmod;
   struct _io_chmod_reply        chmod_reply;
   struct _io_chown              chown;
   struct _io_chown_reply        chown_reply;
   struct _io_fstat              fstat;
   struct _io_fstat_reply        fstat_reply;
   
   struct _das_msg               das_msg;
   struct _das_query             das_query;
   struct _das_query_reply       das_query_reply;
   struct _das_alloc             das_alloc;
   struct _das_alloc_reply       das_alloc_reply;
   
   // Analog to Digital
   struct _das_read              das_ad_sync;
   struct _das_read_reply        das_ad_sync_reply;
   struct _das_read              das_ad_enque;
   struct _das_read_reply        das_ad_enque_reply;
   struct _das_read              das_ad_read;            
   struct _das_read_reply        das_ad_read_reply;
   struct _das_term              das_ad_term;
   struct _das_term_reply        das_ad_term_reply;
   struct _das_scan              das_ad_scan;
   struct _das_scan_reply        das_ad_scan_reply;
   struct _das_s_rate            das_ad_sam;
   struct _das_s_rate_reply      das_ad_sam_reply;
   struct _das_set_trig          das_ad_set_trig;
   struct _das_set_trig_reply    das_ad_set_trig_reply;
   struct _das_get_trig          das_ad_get_trig;
   struct _das_get_trig_reply    das_ad_get_trig_reply;
   struct _das_set_range         das_ad_set_range;
   struct _das_set_range_reply   das_ad_set_range_reply;
   struct _das_get_range         das_ad_get_range;
   struct _das_get_range_reply   das_ad_get_range_reply;
   struct _das_get_queue         das_ad_get_queue;
   struct _das_get_queue_reply   das_ad_get_queue_reply;

   // Digital to Analog
   struct _das_write             das_da_sync;
   struct _das_write_reply       das_da_sync_reply;
   struct _das_write             das_da_enque;
   struct _das_write_reply       das_da_enque_reply;
   struct _das_da_wait           das_da_wait;
   struct _das_da_wait_reply     das_da_wait_reply;
   struct _das_s_rate            das_da_sam;
   struct _das_s_rate_reply      das_da_sam_reply;
   struct _das_set_trig          das_da_set_trig;
   struct _das_set_trig_reply    das_da_set_trig_reply;
   struct _das_get_trig          das_da_get_trig;
   struct _das_get_trig_reply    das_da_get_trig_reply;
   struct _das_term              das_da_term;
   struct _das_term_reply        das_da_term_reply;
   struct _das_set_range         das_da_set_range;
   struct _das_set_range_reply   das_da_set_range_reply;
   struct _das_get_range         das_da_get_range;
   struct _das_get_range_reply   das_da_get_range_reply;
   struct _das_get_queue         das_da_get_queue;
   struct _das_get_queue_reply   das_da_get_queue_reply;

   // Digital I/O
   struct _das_dout              das_dout;
   struct _das_dout_reply        das_dout_reply;
   struct _das_din               das_din;
   struct _das_din_reply         das_din_reply;
};

#define _DAS_STRUCT
#endif
