   #include <stdio.h>
   #include <stdlib.h>
   #include <unistd.h>
   #include <fcntl.h>
   #include <sys/proxy.h>
   #include <das.h>
   
   void main()
   {
      int fd, conv;
      das_buf_t *buf;
      short int *cbufp;
      pid_t proxy;

      /* open the 'das' acquisition manager/library */
      if ((fd = das_open("/dev/das", O_WRONLY, NULL)) == -1)
      {
         perror("das_open");
         exit(EXIT_FAILURE);
      }

      if ((buf = das_alloc(fd, 1, 200, 0)) == NULL)
      {
         perror("das_alloc");
         exit(EXIT_FAILURE);
      }

      if((proxy = qnx_proxy_attach(0, 0, 0, -1)) == -1)
      {
         perror("qnx_proxy_attach");
         exit(EXIT_FAILURE);
      }

      printf("das_da_enqueue with interrupts\n");        
      printf("Output 0->4095 to D/A channel 0\n");

      cbufp = (short int *)buf->bufp;

      /* Fill D/A buffer with data to write out.      */
      for (conv = 0; conv < 100; conv++)
         cbufp[conv] = conv*(4095/100);

      /* set the conversion rate to 50000 clock ticks on channel 0*/
      if (das_da_set_sample(fd, 0, 50000) == -1)
      {
         perror("das_da_set_sample");
         exit(EXIT_FAILURE);
      }
      
      /* perform a 100 D/A conversions on channel 0      */
      if (das_da_enqueue(fd, 0, buf, 200, DAS_SNGL_CYCLE | DAS_INT, proxy) == -1)
      {
         perror("das_da_enqueue");
         exit(EXIT_FAILURE);
      }

      /* see if the conversions are complete          */       
      while (das_da_cwait(fd, 0, 0) == NULL)
      {
         /* do something useful while waiting         */
         sleep(1);
      }

      printf("Done!\n");
            
      if (das_da_term(fd, 0) == -1)
      {
         perror("das_da_term");
         exit(EXIT_FAILURE);
      }
      das_free(fd, buf);

      /* close the acquisition manage/library            */
      if (das_close(fd) == -1)
      {
         perror("das_close");
         exit(EXIT_FAILURE);
      }

      qnx_proxy_detach(proxy);
      exit(EXIT_SUCCESS);
   }
