//*****************************************************************************
// This demo has been provided as an example which uses most of the QNX
// data-aquisition (das) library functions.
// All driver functions have been group into 4 files which are:
//          demo.c      - open/close.
//          demo_aout.c - D/A operations.
//          demo_ain.c  - A/D operations.
//          demo_dig.c  - digita I/O.
//
// The A/D routines have been setup for double buffering and the D/A routines
// have been setup for dual channel operation, as well as performing A/D
// polling (das_ad_poll) operations at the same time.
//*****************************************************************************
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <fcntl.h>
#include <i86.h>
#include <stddef.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <das.h>
#include <sys/proxy.h>

#define PI 3.1415
#define ESC 0x1b

extern void analog_in(void);
extern void dig_io(void);
extern void analog_out(void);

void init_das(int argc, char *argv[]);

das_info info;
int fd, num_ai;
pid_t proxy, tproxy;
nid_t node;

void main(int argc, char *argv[])
{
   int  choice;
      
   init_das(argc, argv);
   
   while(choice != ESC)
   {
      puts("");
      printf("\n\n\n\n**********************************************************");
      printf("**********************\n");
      puts("\n                     Select the type of operation to perform.\n");
      puts("                                  1> Digital I/O.\n");
      puts("                                  2> Analog Output.\n");
      puts("                                  3> Analog Input.\n");
      puts("                                ESC> QUIT.\n");
      printf("\n**********************************************************");
      printf("**********************\n");   
      fprintf(stderr, "Enter choice :> ");
      choice = getch();
      switch(choice)
      {
         case  '1':  dig_io();
                     break;
         
         case  '2':  analog_out();
                     break;
      
         case  '3':  analog_in();
                     break;
         
         case  ESC:  das_close(fd);
                     break;
      }
   }
   dinit_das();
   exit(0);
}

void dinit_das()
{
   qnx_proxy_detach(tproxy);
   qnx_proxy_rem_detach(node, proxy);
   das_close(fd);
}


//*****************************************************************************
void init_das(int argc, char *argv[])
{
   int opt;
   char path[255];
   char prefix[20] = {0};

   node = getnid();   
                    // Parse command line for arguments.
   while((opt = getopt(argc, argv, "n:P:")) != -1)
   {

      switch(opt)
      {
         case 'P':   // Prefix
            strcpy(prefix, optarg);
            break;
         
         case 'n':   // node
            node = atoi(optarg);
            break;
      }
   }
   
   if(prefix[0] == NULL)
      sprintf(path, "//%d/dev/das", node);
   else
      sprintf(path, "//%d/dev/%s", node, prefix);
   
   if ((fd = das_open(path, O_RDWR , NULL)) == -1)
   {
      perror("das_open");
      exit(-1);
   }
   das_get_info(fd, &info);
   num_ai = (info.chnl_cnfg == DAS_DIFFERENTIAL) ? 8 : 16;
   tproxy = qnx_proxy_attach(0,0,0,-1);
   proxy = qnx_proxy_rem_attach(node, tproxy);
}
