#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>
#include <unistd.h>
#include <conio.h>
#include <sys/seginfo.h>
#include <sys/proxy.h>
#include <sys/kernel.h>

#include <das.h>

#define DMA    1
#define INT    2
#define ESC    0x1b
#define VDIV   1000000L

struct   ranges
{
   long  rmin;
   long  rmax;
};

struct ranges rng[11] = {
          {         0, 10000000 },        //    0V - 10V
          {         0,  5000000 },        //    0V - 5V
          {         0,  2000000 },        //    0V - 2V
          {         0,  1000000 },        //    0V - 1V
          {         0,   100000 },        //    0V - 100mV
          { -10000000, 10000000 },        //  -10V - 10V
          {  -5000000,  5000000 },        //   -5V - 5V
          {  -2500000,  2500000 },        // -2.5V - 2.5V
          {  -1000000,  1000000 },        //   -1V - 1V
          {   -500000,   500000 },        // -0.5V - 0.5V
          {    -50000,    50000  } };     // -50mV - 50mV

extern pid_t proxy;
extern int num_ai, fd;
extern das_info info;

void ad_poll(void);
void ad_int_dma(int mode);
void ad_dmap(void);
void ad_sample(void);
void set_blk(void);
void ad_srng(void);
void ad_grng(void);
void ad_strig(void);
void ad_gtrig(void);
extern float convert_to_volts(unsigned int data, long rmin, long rmax);

void analog_in(void)
{
   int  choice;
   
   while(choice != ESC)
   {
      puts("");
      printf("\n\n\n**********************************************************");
      printf("**********************\n");
      puts("\n                     Select the function to perform.\n");
      puts("                                  1>  das_ad_sync()");
      puts("                                  2>  das_ad_enqueue() - INT");
      puts("                                  3>  das_ad_enqueue() - DMA");
      puts("                                  4>  das_ad_enqueue() - DMA_SYNC");
      puts("                                  5>  das_ad_set_sample()");
      puts("                                  6>  das_set_scan()");
      puts("                                  7>  das_ad_set_range()");
      puts("                                  8>  das_ad_get_range()");
      puts("                                ESC>  Return to Main Menu.\n");
      printf("\n**********************************************************");
      printf("**********************\n");   
      fprintf(stderr, "Enter choice :> ");
      choice = getch();
      switch(choice)
      {
         case  '1':  ad_poll();
                     break;
         
         case  '2':  ad_enqueue(DAS_INT);
                     break;
      
         case  '3':  ad_enqueue(DAS_DMA);
                     break;
         
         case  '4':  ad_enqueue(DAS_DMA_SYNC);
                     break;
                  
         case  '5':  ad_sample();
                     break;
                  
         case  '6':  set_blk();
                     break;
                  
         case  '7':  ad_srng();
                     break;
         
         case  '8':  ad_grng();
                     break;
      }
   }

}

//*****************************************************************************
void ad_poll(void)
{
   int chnl;
   short int ad_data;
   struct ranges ad_rng[16];
   float volts;
   
   puts("das_ad_sync()\n");

   for(chnl = 0; chnl < num_ai; chnl++)
      das_ad_get_range(fd, chnl, &ad_rng[chnl].rmin, &ad_rng[chnl].rmax);

   while(!kbhit())
   {
      for(chnl=0;chnl<num_ai;chnl++)
      {
         if (das_ad_sync(fd, chnl, (char *)&ad_data) == -1)
         {
            perror("das_ad_sync");
            break;    
         }
         volts = convert_to_volts(ad_data>>4, ad_rng[chnl].rmin, ad_rng[chnl].rmax);
         printf("channel : %d = % 2.2f\n",ad_data&0xf, volts);
      }
      puts("=! ");
   }
   getch();
}

//*****************************************************************************
void ad_enqueue(int oper)
{
   das_buf_t *buf;
   short int *cbuf;
   struct ranges ad_rng[16];
   int bufp=0, chnl, entry;
   float volts;
   
   if(oper==DAS_INT)
      puts("das_ad_enqueue DAS_INT\n");
   else if(oper==DAS_DMA)
      puts("das_ad_enqueue DAS_DMA\n");
   else
      puts("das_ad_enqueue DAS_DMA_SYNC\n");
   
   if ((buf = das_alloc(fd, 2, 32, _PMF_DMA_SAFE)) == NULL)
   {
      perror("das_alloc");
      exit(-1);
   }
      
   for(chnl = 0; chnl < num_ai; chnl++)
      das_ad_get_range(fd, chnl, &ad_rng[chnl].rmin, &ad_rng[chnl].rmax);

   if ((cbuf = (short int *)malloc(32)) == NULL)
   {
      perror("malloc");
      exit(-1);
   }

   if (das_ad_enqueue(fd, 0, &buf[0], num_ai * info.ad_res , DAS_SNGL_CYCLE | oper, proxy) == -1)
   {
      perror("das_ad_enqueue");
      getch();
   }

   if(oper != DAS_DMA_SYNC)
      if (das_ad_enqueue(fd, 0, &buf[1], num_ai * info.ad_res, DAS_SNGL_CYCLE | oper, proxy) == -1)
      {
         perror("das_ad_enqueue");
         getch();
      }
      
   while(!kbhit())
   {
      if (das_ad_read(fd, (char *)cbuf, num_ai * info.ad_res, 0, 0) == -1)
      {
         perror("das_ad_read");
         break;
      }
   
      for(entry = 0; entry < num_ai; entry++)
      {
         chnl = cbuf[entry] & 0xf;
         volts = convert_to_volts(cbuf[entry]>>4, ad_rng[chnl].rmin, ad_rng[chnl].rmax);
         printf("channel %d = % 2.2f\n", cbuf[entry] & 0xf ,volts);

      } 
      puts("=! ");
      if (das_ad_enqueue(fd, 0, &buf[bufp], num_ai * info.ad_res, DAS_SNGL_CYCLE | ((oper == DAS_DMA) ?
            DAS_DMA : DAS_INT), proxy) == -1)
      {
         perror("das_ad_enqueue");
         getch();
      }
      if(oper != DAS_DMA_SYNC)
         if(++bufp > 1)
            bufp=0;
   }
   das_ad_term(fd, 0);
   free(cbuf);
   das_free(fd, buf);
   getch();      
}

//*****************************************************************************
void ad_sample(void)
{
   char s_rate_buf[10];
   long ticks;
   
   puts("das_ad_set_sample\n");
   printf("Enter sample rate in ticks:> ");
   ticks = atol(fgets(s_rate_buf,sizeof(s_rate_buf),stdin));
   while (das_ad_set_sample(fd, 0, ticks) == -1)
   {
      perror("das_ad_set_sample");
      printf("<try again> : Enter sample rate in ticks:> ");
      ticks = atol(fgets(s_rate_buf,sizeof(s_rate_buf),stdin));
   }
}

//*****************************************************************************
void set_blk(void)
{
   char seq[8] = {0}, cnt;
   char key=0, buf[9];

   puts("das_set_scan\n");   
   for(cnt=0;cnt<8;cnt++)
   {
      fprintf(stderr, "Enter channel # :> ");
      seq[cnt] = atoi(fgets(buf, sizeof(buf), stdin));
      fprintf(stderr, "Add another entry (Y or N) :> ");
      if((key = toupper(getch())) == 'N')
         break;
      printf("\n");
   }
   fprintf(stderr, "\nSet scan sequence (Y or N) :> ");
   if((key = toupper(getch())) == 'Y')
      das_ad_set_scan(fd, &seq, cnt+1);
}

//*****************************************************************************      
void ad_srng(void)
{
   char buf[9], chnl, range;

   puts("das_ad_set_range()\n");   
   puts("0  >   0 to +10V");
   puts("1  >   0 to +5V");
   puts("2  >   0 to +2V");
   puts("3  >   0 to +1V");
   puts("4  >   0 to 100mV");
   puts("5  >   +-10V");
   puts("6  >   +-5V");
   puts("7  >   +-2.5V");
   puts("8  >   +-1V");
   puts("9  >   +-0.5V");
   puts("10 >   +-50mV");
 
   printf("Enter channel # :> ");
   chnl = atoi(fgets(buf, sizeof(buf), stdin));
   printf("Enter range :> ");
   range = atoi(fgets(buf, sizeof(buf), stdin));
   if (das_ad_set_range(fd, chnl, rng[range].rmin, rng[range].rmax) == -1)
   {
      perror("das_ad_set_range");
      getch();
   }
}

//*****************************************************************************
void ad_grng(void)
{
   int i;
   long rmin, rmax;
   
   puts("das_ad_get_range()\n");
   for (i = 0; i <num_ai; i++)
      if (das_ad_get_range(fd, i, &rmin, &rmax) == -1)
         perror("das_ad_get_range");
      else
         printf("Channel # %d : Range = %ld to %ldV \n", i, rmin/VDIV, rmax/VDIV);
   getch();
}

