#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <math.h>
#include <das.h>
#include <sys/seginfo.h>
#include <sys/proxy.h>
#include <sys/kernel.h>

#define DMA    1
#define INT    2
#define ESC    0x1b
#define PGUP   0xa2
#define PGDN   0xaa
#define PI     3.1415
#define VDIV   1000000L

struct   ranges
{
   long  rmin;
   long  rmax;
};

struct da
{ 
#if _ADAPTER == _MET_DAS_20
   short int data       : 12;
   short int not_used   : 4;
#else
   short int not_used   : 4;
   short int data       : 12;
#endif
};

#ifdef __386__
   typedef struct da da_t;
#else
   typedef struct da far da_t;
#endif

extern struct ranges rng[11];
extern int fd;
extern pid_t proxy;

void da_poll(void);
void da_enqueue(int oper);
void da_dmap(void);
void da_sample(void);
void da_srng(void);
void da_grng(void);
void da_strig(void);
void da_gtrig(void);
float convert_to_volts(unsigned data, long rmin, long rmax);

void analog_out(void)
{
   int  choice;
   
   while(choice != ESC)
   {
      puts("");
      printf("\n\n\n**********************************************************");
      printf("**********************\n");
      puts("\n                     Select the function to perform.\n");
      puts("                              1>  das_da_sync()");
      puts("                              2>  das_da_enqueue() - INT");
      puts("                              3>  das_da_enqueue() - DMA");
      puts("                              4>  das_da_enqueue() - DMA_SYNC");
      puts("                              5>  das_da_set_sample()");
      puts("                              6>  das_da_set_range()");
      puts("                              7>  das_da_get_range()");
      puts("                            ESC>  Return to Main Menu.\n");
      printf("\n**********************************************************");
      printf("**********************\n");   
      fprintf(stderr, "Enter choice :> ");
      choice = getch();
      switch(choice)
      {
         case  '1':  da_poll();
                     break;
         
         case  '2':  da_enqueue(DAS_INT);
                      break;
      
         case  '3':  da_enqueue(DAS_DMA);
                     break;
         
         case  '4':  da_enqueue(DAS_DMA_SYNC);
                     break;
                  
         case  '5':  da_sample();
                     break;
                  
         case  '6':  da_srng();
                     break;
         
         case  '7':  da_grng();
                     break;
      }
   }
}

//*****************************************************************************
void da_poll(void)
{
   da_t da_data;
   short int ad_data;
   int chnl, key;
   struct ranges rng[2];
   float volts;
   
   puts("das_da_sync\n\n");
   puts("Connect Analog output channel # 0 to Analog input channel #0");
   puts("& Analog output channel #1 to Analog input channel #1");
   puts("\nUse +/- to adjust analog output by increments of 1");
   puts("Use Pgup/Pgdn to adjust analog output by increments of 10");
   puts("Select Output channel 0 or 1");
   puts("ESC - Quit\n\n");

   if (das_da_get_range(fd, 0, &rng[0].rmin, &rng[0].rmax) == -1)
   {
      rng[0].rmin = 0;
      rng[0].rmax = 10000000;
   }
   if (das_da_get_range(fd, 1, &rng[1].rmin, &rng[1].rmax) == -1)
   {
      rng[1].rmin = 0;
      rng[1].rmax = 10000000;
   }
      
   das_ad_set_range(fd, 0, rng[0].rmin, rng[0].rmax);
   das_ad_set_range(fd, 1, rng[1].rmin, rng[1].rmax);   

   da_data.data = 0;
   chnl = 0;
   while( key != ESC )
   {
      if (das_da_sync(fd, chnl , (char *)&da_data) == -1)
      {
         perror("das_da_sync");
         break;
      }
      fprintf(stderr, "Output Channel : %d Output : %#04x \n", chnl, da_data.data&0x0fff);
      if (das_ad_sync(fd, 0, (char *)&ad_data) == -1)
      {
         perror("das_ad_sync 1");
         break;    
      }
      volts = convert_to_volts(ad_data>>4, rng[0].rmin, rng[0].rmax);
      fprintf(stderr,"Input Channel #0 : %2.2f V\n", volts);     
      if (das_ad_sync(fd, 1, (char *)&ad_data) == -1)
      {
         perror("das_ad_sync 2");
         break;    
      }
      volts = convert_to_volts(ad_data>>4, rng[1].rmin, rng[1].rmax);
      fprintf(stderr,"Input Channel #1 : %2.2f V=, ", volts);

      key=0;
      if (kbhit())
         key = getch();
      switch(key)
      {
         case '+':
            ++(da_data.data);
            break;
            
         case '-':
            --(da_data.data);
            break;

         case '0':
            chnl = 0;
            break;
            
         case '1':
            chnl = 1;
            break;
            
         case 0xff:
            switch(getch())
            {
               case PGUP:
                  (da_data.data)+=10;
                  break;
                 
               case PGDN:
                  (da_data.data)-=10;
                  break;
            }
      }
   }
}

//*****************************************************************************
void da_enqueue(int oper)
{
   das_buf_t *buf;
   da_t *bptr;
   short int ad_data;
   int i = 0;
   struct ranges rng[2];
   float f_val, volts;

   if(oper==DAS_INT)
      puts("das_da_enqueue DAS_INT\n");
   else if(oper==DAS_DMA)
      puts("das_da_enqueue DAS_DMA\n");
   else
      puts("das_da_enqueue DAS_DMA_SYNC\n");

   if (das_da_get_range(fd, 0, &rng[0].rmin, &rng[0].rmax) == -1)
   {
      rng[0].rmin = 0;
      rng[0].rmax = 10000000;
   }
   if (das_da_get_range(fd, 1, &rng[1].rmin, &rng[1].rmax) == -1)
   {
      rng[1].rmin = 0;
      rng[1].rmax = 10000000;
   }
      
   das_ad_set_range(fd, 0, rng[0].rmin, rng[0].rmax);
   das_ad_set_range(fd, 1, rng[1].rmin, rng[1].rmax);   

   if ((buf = das_alloc(fd, 1, 1000, _PMF_DMA_SAFE)) == NULL)
   {
      perror("das_alloc");
      exit(-1);
   }
   
   bptr = (da_t *)buf->bufp;
   
   // Make a sinwave
   for (i = 0, f_val = 0; i < 500; i+=2)
   {
      bptr[i].data = (2048.0 * sin(f_val));
      bptr[i+1].data = (2048.0 * cos(f_val));
      f_val+=((8*PI)/1000.0);
   }            
   if (das_da_enqueue(fd, 0, buf, 1000, DAS_SNGL_CYCLE | DAS_DUAL_CHNL
                            | oper, proxy) == -1)
   {
      perror("das_da_enqueue");
      return;
   }
   i=0;
   while(!kbhit())
   {
      fprintf(stderr,"=# %d sine wave to chnl -> 0 & 1\n", ++i);
     Creceive(proxy, 0, 0);
      while(das_da_cwait(fd, 0, DAS_REUSE) == 0)
      {
         if (das_ad_sync(fd, 0, (char *)&ad_data) == -1)
         {
            perror("das_ad_sync 1");
            break;    
         }
         volts = convert_to_volts(ad_data>>4, rng[0].rmin, rng[0].rmax);
         fprintf(stderr,"Input Channel #0 : % 3.2f V\n", volts);
         if (das_ad_sync(fd, 1, (char *)&ad_data) == -1)
         {
            perror("das_ad_sync 2");
            break;    
         }
         volts = convert_to_volts(ad_data>>4, rng[1].rmin, rng[1].rmax);
         fprintf(stderr,"Input Channel #1 : % 3.2f V=$ ", volts);
      }
   }
   getch();
   das_da_term(fd, 0);
   das_free(fd, buf);
}

//*****************************************************************************
void da_sample(void)
{
   char s_rate_buf[10];
   long ticks;
   
   puts("das_da_set_sample\n");
   printf("Enter sample rate in ticks:> ");
   ticks = atol(fgets(s_rate_buf,sizeof(s_rate_buf),stdin));
   while (das_da_set_sample(fd, 0, ticks) == -1)
   {
      perror("das_da_set_sample");
      printf("<try again> : Enter sample rate in ticks:> ");
      ticks = atol(fgets(s_rate_buf,sizeof(s_rate_buf),stdin));
   }
}

//*****************************************************************************
void da_srng(void)
{
   char buf[9], chnl, range;

   puts("das_da_set_range()\n");   
   puts("0  >   0 to +10V");
   puts("1  >   0 to +5V");
   puts("2  >   0 to +2V");
   puts("3  >   0 to +1V");
   puts("4  >   0 to 100mV");
   puts("5  >   +-10V");
   puts("6  >   +-5V");
   puts("7  >   +-2.5V");
   puts("8  >   +-1V");
   puts("9  >   +-0.5V");
   puts("10 >   +-50mV");
 
   printf("Enter channel # :> ");
   chnl = atoi(fgets(buf, sizeof(buf), stdin));
   printf("Enter range :> ");
   range = atoi(fgets(buf, sizeof(buf), stdin));
   if (das_da_set_range(fd, chnl, rng[range].rmin, rng[range].rmax) == -1)
   {
      perror("das_da_set_range");
      getch();
   }
}

//*****************************************************************************
void da_grng(void)
{
   int i;
   long rmin, rmax;

   puts("das_da_get_range()\n");
   for (i = 0; i <2; i++)
      if (das_da_get_range(fd, i, &rmin, &rmax) == -1)
         perror("das_da_get_range");
      else
         printf("Channel # %d : Range = %ld to %ldV \n", i, rmin/VDIV, rmax/VDIV);
   getch();
}

//*****************************************************************************
float convert_to_volts(unsigned int data, long rmin, long rmax)
{
   float volts;
   
   rmin/=VDIV;
   rmax/=VDIV;

#if _ADAPTER == _MET_DAS_20
   if (rmin < 0)
      volts = (float)((int)data) / 4096.0 * (rmax - rmin);
   else 
   {
      data &= 0x0fff;
      volts = ((float)data) / 4096.0 * (rmax - rmin);
   }     
      
#else
   data &= 0x0fff;
   if (rmin < 0)
      volts = ((float)((int)data-2047)) / 4096.0 * (rmax - rmin);
   else
      volts = ((float)data) / 4096.0 * (rmax - rmin);
#endif
   return(volts);
}
