// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// das_msg.h contains defines for message sub-types and message
// structure definitions.
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#include <sys/types.h>
#include <sys/sendmx.h>

#include <das.h>
                                       // DAS library I/O message
#define _IO_DAS               0x0400
                                       // DAS library I/O message sub-types
#define _DAS_AD_SYNC          0x0001
#define _DAS_AD_ENQUE         0x0002
#define _DAS_AD_READ          0x0003
#define _DAS_AD_CREAD         0x0004
#define _DAS_AD_TERM          0x0005
#define _DAS_AD_SET_SMP       0x0006
#define _DAS_AD_SET_SCAN      0x0007
#define _DAS_AD_GET_RANGE     0x0008
#define _DAS_AD_SET_RANGE     0x0009
#define _DAS_AD_SET_TRIG      0x000A
#define _DAS_AD_GET_TRIG      0x000B
#define _DAS_AD_GET_RDY_QUE   0x000C
#define _DAS_AD_GET_FIN_QUE   0x000D

#define _DAS_DA_SYNC          0x000E                                  
#define _DAS_DA_ENQUE         0x000F
#define _DAS_DA_WAIT          0x0010
#define _DAS_DA_CWAIT         0x0011
#define _DAS_DA_TERM          0x0012
#define _DAS_DA_SET_SMP       0x0013
#define _DAS_DA_SET_RANGE     0x0014
#define _DAS_DA_GET_RANGE     0x0015
#define _DAS_DA_SET_TRIG      0x0016
#define _DAS_DA_GET_TRIG      0x0017
#define _DAS_DA_GET_RDY_QUE   0x0018
#define _DAS_DA_GET_FIN_QUE   0x0019
                            
#define _DAS_DIG_IN           0x001A
#define _DAS_DIG_OUT          0x001B

#define _DAS_QUERY            0x001C
#define _DAS_ALLOC            0x001D
#define _DAS_FREE             0x001E

struct _das_msg
{
   msg_t             type;
   short int         fd,
                     request;
};

struct _das_read     // For enqueue & read requests.
{
   msg_t             type;
   short int         fd,
                     request,
                     mode,
                     chnl;
   short unsigned    n_bytes;
   mpid_t            proxy;
};

struct _das_read_reply
{
   msg_t             status;
   short unsigned    n_bytes;
   char              data[1];
};

struct _das_write    // All D/A requests.
{
   msg_t             type;
   short int         fd,
                     request,
                     mode,
                     chnl;
   short unsigned    n_bytes;
   mpid_t            proxy;
   char              data[1];
};

struct _das_write_reply
{
   msg_t             status;
   short unsigned    n_bytes;
};

struct _das_query    // Query. (Sent in das_open).
{
   msg_t             type;
   short int         fd,
                     request;
};

struct _das_query_reply
{
   msg_t             status;
   das_info          d_info;
};

struct _das_alloc
{
   msg_t             type;
   short int         fd,
                     request,
                     flags;
   short unsigned    n_bufs,
                     n_bytes;
};

struct _das_alloc_reply
{
   msg_t             status;
};

struct _das_scan     // Set scan block.
{
   msg_t             type;
   short int         fd,
                     request,
                     n_entry;
   char              data[1];
};

struct _das_scan_reply
{
   msg_t             status;
};

struct _das_get_range // Get range.
{
   msg_t             type;
   short int         fd,
                     request,
                     chnl;
};

struct _das_get_range_reply
{
   msg_t             status;
   long              range_min,
                     range_max;
};

struct _das_set_range // Set range.
{
   msg_t             type;
   short int         fd,
                     request,
                     chnl;
   long              range_min,
                     range_max;
};

struct _das_set_range_reply
{
   msg_t             status;
};

struct _das_term     // Terminate.
{
   msg_t             type;
   short int         fd,
                     request,
                     chnl;
};

struct _das_term_reply 
{
   msg_t             status;
};

struct _das_set_trig // Set trigger source.
{
   msg_t             type;
   short int         fd,
                     request,                   
                     trig_src,
                     chnl;
};

struct _das_set_trig_reply
{
   msg_t             status;
};

struct _das_get_trig // Get trigger source.
{
   msg_t             type;
   short int         fd,
                     request,
                     chnl;
};

struct _das_get_trig_reply
{
   msg_t             status;
   short int         trig_src;
};

struct _das_s_rate   // Change sample rate.
{
   msg_t             type;
   short int         fd,
                     request,
                     chnl;
   unsigned long     ticks;
};

struct _das_s_rate_reply
{
   msg_t             status;
};

struct _das_din      // Digital input.
{
   msg_t             type;
   short int         fd,
                     request,
                     port;
};

struct _das_din_reply
{
   msg_t             status;
   long              bits;
};

struct _das_dout     // Digital output.
{
   msg_t             type;
   short int         fd,
                     request,
                     port;
   long              mask,
                     bits;
};

struct _das_dout_reply
{
   msg_t             status;
   long              old_bits,
                     new_bits;
   
};

struct _das_get_queue
{
   msg_t             type;
   short int         fd,
                     request,
                     chnl;
};

struct _das_get_queue_reply
{
   msg_t             status;
   short int         n_ent;
};   

struct _das_da_wait
{
   msg_t             type;
   short int         fd,
                     request,
                     chnl,
                     reuse;
};

struct _das_da_wait_reply
{
   msg_t             status;
   short unsigned    n_bytes;
};

