// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// das_struct.h contains structure definitions for the direct
// library.
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#define  _MET_DAS_20       0x00
#define  _PCL_818          0x01

#ifndef _DAS_STRUCT_H_INCLUDED
#if _ADAPTER == _MET_DAS_20
   #include <met_20.h>
#else
   #include <pcl_818.h>
#endif

#include <sys/types.h>
#include <das_types.h>
#include <das_tmr.h>

struct   das_rate                      // conversion rate in ticks is broken
{                                      // down into two divisors so they can
   int   div1;                         // be directly written out to counters
   int   div2;
};

struct   das_rng                       // contains a minimum and maximum range
{                                      // in microvolts,  an array of structs
   long  min;                          // defining valid range settings for
   long  max;                          // for an adapter is maintianed
};

struct   das_scan                      // defines scan queue
{
   char           scan[QUEUE_SIZE];    // scan queue
   int            last;                // last entry in the queue
};

// das_t data type for D/A and A/D functions
#ifndef _DAS_T_DEFINED_
#ifdef __386__
   typedef  unsigned short int      das_t;
#else
   typedef  unsigned short int far  das_t;
#endif
   #define  _DAS_T_DEFINED_
#endif

struct das_queue
{
   struct   das_queue   *nptr;         // Pointer to next entry in the list
   das_buf_t            *dma;          // Physical address for DMA xfers
   int                  bytes;         // Number of conversions to perform
   das_t                *cp;           // current pointer within buffer.
   int                  cnt;           // conversions so far.
   int                  mode;          // mode
   int                  rtype;         // request type
   pid_t                proxy;         // request complete proxy
};

struct das_chnl                        // used for A/D and D/A
{
   int                  stat;          // status of channel
   int                  trig_src;      // pacer or external trigger
   int                  addr;          // offset from base address
   int                  rng;           // range/gain for channel
   int                  n_rent;        // number of entries in ready queue
   int                  n_fent;        // number of entries in finish queue
   struct   das_queue   *f_hd;         // head of finish queue
   struct   das_queue   *f_tl;         // tail of finish queue
   struct   das_queue   *r_hd;         // head of ready queue
   struct   das_queue   *r_tl;         // tail of ready queue
   struct   das_rate    s_rate;        // conversion rate
};

struct das_port
{
   long           bits;                // current level on port
   int            addr;                // offset from base address
};

struct docb                            // DAS operation control block
{
   das_info             info;                // Adapter configuration
   struct   das_chnl    da[DA_CHNLS];        // Array of D/A channels
   struct   das_chnl    ad[AD_CHNLS];        // Array of A/D channels
   struct   das_port    din[NUM_IPORTS];     // digital I/O port specifics
   struct   das_port    dout[NUM_OPORTS];    // digital I/O port specifics
   cntrs_t              cntrs;               // counter/timer specifics
};

#define _DAS_STRUCT_H_INCLUDED
#endif
