// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// das_tmr.h contains defines and data structures for the Intel
// 8254 and AMD 9513 counter/timer chips.
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#ifndef _DAS_TMR_H_INCLUDED

// defines for AMD 9513 master mode register
#define  _TC_NM1        0x00
#define  _SRC1          0x01           // hardware source pins
#define  _SRC2          0x02
#define  _SRC3          0x03
#define  _SRC4          0x04
#define  _SRC5          0x05
#define  _GATE1         0x06           // hardware gate pins
#define  _GATE2         0x07
#define  _GATE3         0x08
#define  _GATE4         0x09
#define  _GATE5         0x0A
#define  _F1            0x0B           // frequency scalar taps
#define  _F2            0x0C
#define  _F3            0x0D
#define  _F4            0x0E
#define  _F5            0x0F

#define  _DIV16         0x00           // Fout dividing ratios
#define  _DIV1          0x01
#define  _DIV2          0x02
#define  _DIV3          0x03
#define  _DIV4          0x04
#define  _DIV5          0x05
#define  _DIV6          0x06
#define  _DIV7          0x07
#define  _DIV8          0x08
#define  _DIV9          0x09
#define  _DIV10         0x0A
#define  _DIV11         0x0B
#define  _DIV12         0x0C
#define  _DIV13         0x0D
#define  _DIV14         0x0E
#define  _DIV15         0x0F

#define  _ON            0x01           // compare 1 or 2 disable
#define  _OFF           0x00           // compare 1 or 2 enable

#define  _TOD_DIS       0x00           // time of day disable
#define  _TOD_50HZ      0x01           // time of day enabled / 5 input
#define  _TOD_60HZ      0x02           // time of day enabled / 6 input
#define  _TOD_100HZ     0x03           // time of day enabled / 10 input

#define  _FOUT_ON       0x00           // FOUT on
#define  _FOUT_OFF      0x01           // FOUT off

#define  _BUS_8         0x00           // 8-bit bus width
#define  _BUS_16        0x01           // 16-bit bus width

#define  _DP_DINC       0x01           // data pointer disable increment
#define  _DP_EINC       0x00           // data pointer enable increment

#define  _BIN           0x00           // binary division
#define  _BCD_DIV       0x01           // BCD division

#define  _DOWN          0x00           // up or down counting
#define  _UP            0x01

// defines for counter register
#define  _OFF_LO_TC     0x00           // inactive, output low
#define  _ACT_HI_TC     0x01           // active high terminal count pulse
#define  _TC_TOG        0x02           // TC toggled
#define  _OFF_OC_TC     0x04           // inactive, output high impedance
#define  _ACT_LO_TC     0x05           // active low terminal count pulse

#define  _MODE_ABC      0x00           // counter modes
#define  _MODE_DEF      0x01
#define  _MODE_GHI      0x02
#define  _MODE_JKL      0x03
#define  _MODE_NO       0x04
#define  _MODE_QR       0x05
#define  _MODE_S        0x06
#define  _MODE_V        0x07

#define  _RISE          0x00           // rising edge
#define  _FALL          0x01           // falling edge

#define  _NO_GATE       0x00           // no gating
#define  _HL_TC_NM1     0x01           // active high TCN-1
#define  _HL_NP1_GATE   0x02           // active high level gate N+1
#define  _HL_NM1_GATE   0x03           // active high level gate N-1
#define  _HL_GATE_N     0x04           // active high level gate N
#define  _LL_GATE_N     0x05           // active low level gate N
#define  _HE_GATE_N     0x06           // active high edge gate N
#define  _LE_GATE_N     0x07           // active low edge gate N

#define  _LD_DPTR       0x00           // load data pointer
// counter commands
#define  _ARM_CTR       0x01           // arm counters
#define  _LD_CTR        0x02           // load counters
#define  _LD_ARM_CTR    0x03           // load and arm counters
#define  _D_S_CTR       0x04           // diasarm and save counters
#define  _S_CTR         0x05           // save counters
#define  _DARM_CTR      0x06           // disarm counters

// main commands
#define  _D_DPTR_SEQ    0xE8           // disable data pointer sequencing
#define  _E_DPTR_SEQ    0xE0           // enable data pointer sequencing
#define  _E_BUS_16      0xEF           // enable 16-bit data bus
#define  _E_BUS_8       0xE7           // enable 8-bit data bus
#define  _GOFF_FOUT     0xEE           // gate off FOUT
#define  _GON_FOUT      0xE6           // gate on FOUT
#define  _D_PREFETCH    0xF9           // disable prefetch for write operations
#define  _E_PREFETCH    0xF8           // enable prefetch for write operations
#define  _MRESET        0xFF           // master reset

#define  _CTR1          0x01           // counter values for commands
#define  _CTR2          0x02
#define  _CTR3          0x04
#define  _CTR4          0x08
#define  _CTR5          0x10

#define  _MODE          0x00           // .elm values
#define  _LOAD          0x01
#define  _HOLD          0x02
#define  _HOLD_CYCLE    0x03

#define  _ALARM1        0x00           // .elm values for control group
#define  _ALARM2        0x01
#define  _MASTER        0x02
#define  _STATUS        0x03

#define  _CTL_GRP       0x07
// AMD9513 specific defines end

// Intel 8254 specific defines start

#define  RW_LM          0x30           // read/write LSB first, then MSB.
#define  RB_MODE        0xC0           // read-back mode
#define  G_STAT         0xE0           // get status
#define  G_CNT          0xD0           // get count
#define  PIT_CTR        0x0C
#define  CTR            0x02
#define  INTERNAL       0x03
#define  EXTERNAL       0x01
#define  BCD            0x01           // binary coded decimal

// Counter modes of operation
#define  INT_ON_TC      0x00     // interrupt on terminal count
#define  PROG_1_SHOT    0x01     // programable one shot
#define  RATE_GEN       0x02     // rate generator
#define  SQ_W_GEN       0x03     // square wave generator
#define  SOFT_T_STB     0x04     // software triggered strobe
#define  HDW_T_STB      0x05     // hardware triggered strobe

// Intel 8254 specific defines end

typedef union              // AMD 9513 counter mode register type
{
   int   all;           // all bits
   int   ctl   :3;      // output control
   int   dir   :1;      // count down or up (0 or 1)
   int   cmd   :1;      // binary or BCD count (0 or 1)
   int   md    :3;      // modes (8254 uses this one only)
   int   src   :4;      // count source selection
   int   edg   :1;      // source edge rising or falling (0 or 1)
   int   gtl   :3;      // gating control
} cntr_mdr_t;

typedef struct
{
   cntr_mdr_t  md;         // mode register type
   int   ld;         // load register (DAS-20 only)
   int   hd;         // hold register (DAS-20 only)
   int         addr;       // offset from base address PCL-818 only (8253)
} cntr_t;


typedef union              // AMD 9513 master mode register type
{
   int      all;        // all bits
   int      tod   :2;   // time of day mode
   int      cp1   :1;   // compare 1 disable or enable (0 or 1)
   int      cp2   :1;   // compare 2 disable or enable (0 or 1)
   int      src   :4;   // FOUT source
   int      div   :4;   // FOUT divider
   int      fgt   :1;   // FOUT gate on or off (0 or 1)
   int      dsz   :1;   // data bus width 8 or 16 bit (0 or 1)
   int      dpc   :1;   // data pointer control enable or disable increment
   int      sc    :1;   // scaler control
} cntr_mmdr_t;

typedef struct
{
   cntr_mmdr_t mtr;        // master mode register type
   cntr_t      ctr[5];     // counter type
   int   al1;        // alarm register 1
   int   al2;        // alarm register 2
   int         pit_stat;   // Contents of counter/timer
                           // enable register (PCL-818 only)
} cntrs_t;

typedef struct                      // data pointer register command
{
   int         grp   :3;   // group
   int         elm   :2;   // element
   int         cmd   :3;   // command code
} dprcmd_t;

typedef struct                      // counter command
{
   int         ctr   :5;   // counter 1=0x1, 2=0x2, 3=0x4, 4=0x8, 5=0x10
   int         cmd   :3;   // command code
} ctrcmd_t;

typedef union
{
   int         cmd;
   ctrcmd_t    ccmd;
   dprcmd_t    dcmd;
} cmd_t;

typedef union              // AMD 9513 status register type
{
   int         all;        // all bits
   int         bp    :1;   // byte pointer
   int         ot_1  :1;   // output 1
   int         ot_2  :1;   // output 2
   int         ot_3  :1;   // output 3
   int         ot_4  :1;   // output 4
   int         ot_5  :1;   // output 5
   int         xxx   :2;   // undefined
} cntr_st_t;

#define _DAS_TMR_H_INCLUDED
#endif
