// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// met_20.h contains defines for the Metrabyte DAS-20 adapter.
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#define  AD_CHNLS       0x10           // number of A/D channels
#define  DA_CHNLS       0x02           // number of D/A channels
#define  QUEUE_SIZE     0x800          // queue ram (2048 bytes)

#define  MIN_ADDR       0x200          // minimum base address
#define  MAX_ADDR       0x3F0          // maximum base address
#define  MIN_DMA_CHNL   0x01           // DMA channels
#define  MAX_DMA_CHNL   0x03
#define  MIN_IRQ        0x02           // IRQ
#define  MAX_IRQ        0x07

#define  MAX_CONV       65535
#define  EOC            0x02           // End of conversion

#define  DIFF_MODE      0x00     
#define  SNGL_MODE      0x20
#define  MAX_DIF_CHNL   0x07
#define  MIN_CHNL       0x00
#define  MAX_SNGL_CHNL  0x0F
#define  MAX_DA_CHNL    0x02
#define  MAX_AD_CHNL    0x10
#define  NUM_OPORTS     0x01
#define  NUM_IPORTS     0x01

#define  COMPLETE       0x00           // status of a channel
#define  IN_PROG        0x01

#define  FALSE          0x00
#define  TRUE           0x01
#define  END_OF_QUEUE   0x01
#define  NUM_AD_RANGES  0x08
#define  NUM_DA_RANGES  0x04
#define  MAX_CTR        0x05        // number of counters in the AMD 9513
#define  MODE_MASK      0xFF
#define  OPER_MASK      0xF00
   // maximum number of entries in the ready and finish queue at a time
#define  MAX_QUEUE_ENT  0x14
#define  MIN_TICKS      0x32        // 50 ticks (5MHz/100kHz)
#define  MAX_TICKS      0xFFFFFFFF
#define  INT_TICKS      0x1F4       // minimum ticks for interrups (10kHz) 

   // I/O Port Address Map
#define  ADC_DATA       0x00
#define  START_ADC      0x00
#define  QUEUE_VIEW     0x02
#define  QUEUE_LOAD     0x02
#define  ADC_MUX_CTL    0x02
#define  ADC_CTL        0x03
#define  INT_STAT_REG   0x04
#define  INT_CTL_REG    0x04
#define  DMA_CTL_REG    0x05
#define  CTR_DATA       0x06
#define  CTR_CTL        0x07
#define  DAC0           0x08
#define  DAC1           0x0A
#define  DIG_IN         0x0B
#define  DIG_OUT        0x0C

   // trigger source bit locations in ADC_CTL register
#define  _SW_TRIG       0x00
#define  _PC_TRIG       0x00
#define  _EX_TRIG       0x04
#define  _HW_TRIG       0x08
#define  _A_LOW         0x00
#define  _A_HIGH        0x01
#define  _DIS_GATE      0x00
#define  _ENB_GATE      0x02

   // Queue modes
#define  QUEUE_STATIC   0x00        // Queue stationary
#define  QUEUE_RW       0x40        // Queue access to be loaded or examined
#define  QUEUE_STOP     0x80        // Queue sequences and stops at end of 
                                    // queue flag
#define  QUEUE_WRAP     0xC0        // Queue sequences and wraps around at
                                    // end of queue flag

   // Interrupt modes
#define  EOC_INT        0x00        // ADC End of Conversion interrupt
#define  TM2_INT        0x04        // Timer #2 Output interrupt
#define  EQU_INT        0x08        // End of Instruction Queue interrupt
#define  DMA_INT        0x0C        // DMA Terminal Count interrupt
#define  INT_MASK       0x0C        // Mask to determine int source

   // D/A DMA transfer modes
#define  ADC_DMA        0x00        // 2-Byte ADC data xfer initiated by EOC
#define  DAC0_DMA       0x10        // 2-Byte data xfer initiated by timer 2 output
#define  DAC1_DMA       0x20        // 2-Byte data xfer initiated by timer 2 output
#define  DAC1_2_DMA     0x30        // 4-Byte data xfer initiated by timer 2 output
#define  XFR_MASK       0x30        // Mask to determine DMA mode

   // Interrupt modes
#define  EOC_INT        0x00        // ADC End of Conversion interrupt
#define  TM2_INT        0x04        // Timer #2 Output interrupt
#define  EQU_INT        0x08        // End of Instruction Queue interrupt
#define  DMA_INT        0x0C        // DMA Terminal Count interrupt
#define  INT_MASK       0x0C        // Mask to determine int source

   // defines for INT_CTL_REG
#define  INT_ENABLE     0x80
#define  INT_DISABLE    0x00

   // defines for ADC_CTL
#define  DMA_UNACTIVE   0x00
#define  DMA_ACTIVE     0x20

   // defines for DMA_CTL_REG
#define  DMA_DISABLE    0x00
#define  DMA_ENABLE     0x80
#define  DMA_CH1        0x00
#define  DMA_CH3        0x40

#define  ADC_DMA        0x00        // 2-Byte ADC data xfer initiated by EOC
#define  DAC0_DMA       0x10        // 2-Byte data xfer initiated by timer 2 output
#define  DAC1_DMA       0x20        // 2-Byte data xfer initiated by timer 2 output
#define  DAC1_2_DMA     0x30        // 4-Byte data xfer initiated by timer 2 output
#define  XFR_MASK       0x30        // Mask to determine DMA mode

   // defines for instrument amplifier gain/ranges (A/D)
                                    // input range    gain  uni/bipolar
#define  _UN_X1          0          // 0 to +10V      x1    unipolar 
#define  _UN_X10         4          // 0 to 1V        x10   unipolar
#define  _UN_X100        6          // 0 to 100mV     x100  unipolar
#define  _BI_X_5         1          // +-10V          x0.5  bipolar
#define  _BI_X1          3          // +-5V           x1    bipolar
#define  _BI_X10         5          // +-0.5V         x10   bipolar
#define  _BI_X100        7          // +-50mV         x100  bipolar

   // defines for instrument amplifier gain/ranges (D/A)
                                    // input range    gain  uni/bipolar
#define  _UN_X1_DA       0          // 0 to +10V      x1    unipolar 
#define  _BI_X_5_DA      3          // +-10V          x0.5  bipolar
#define  _BI_X1_DA       1          // +-5V           x1    bipolar

   // defines for das_set_mux
#define  SET_ALL_CHNLS  0x00
#define  SET_SNGL_CHNL  0x01

   // defines for configuration file
#define  CFG_FILE          "cfg_default.met"
#define  CFG_FILE_PATH     "/usr/lib/das/config/"
