// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// pcl_818.h contains defines for the Advantech PCL-818 adapter.
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#define  AD_CHNLS       0x10           // number of A/D channels
#define  DA_CHNLS       0x02           // number of D/A channels
#define  QUEUE_SIZE     0x10           // size of scan queue

#define  MIN_ADDR       0x200
#define  MAX_ADDR       0x3F0
#define  MIN_DMA_CHNL   0x01
#define  MAX_DMA_CHNL   0x03
#define  MAX_DIF_CHNL   0x07
#define  MIN_CHNL       0x00
#define  MAX_SNGL_CHNL  0x0F
#define  MAX_DA_CHNL    0x02
#define  MAX_AD_CHNL    0x10
#define  MIN_IRQ        0x02
#define  MAX_IRQ        0x07
#define  MAX_CONV       65535
#define  MAX_CTR        0x03
#define  NUM_OPORTS     0x02
#define  NUM_IPORTS     0x02

   // maximum number of entries in the ready and finish queue at a time
#define  MAX_QUEUE_ENT  0x14
#define  MIN_TICKS      0x0A
#define  MAX_TICKS      0xFFFFFFFFL
#define  INT_TICKS      0x1F4       // minimum ticks for interrups (10kHz) 

   // Triggering sources
#define SW_TRIG         0x01
#define EX_TRIG         0x02
#define PC_TRIG         0x04
   // gateing is enabled by bitwise OR-ing ENB_GATE with a trigger source
#define ENB_GATE        0x08

#define _SW_TRIG        0x01
#define _EX_TRIG        0x02
#define _PC_TRIG        0x03
#define _ENB_GATE       0x08

#define  DIS_PACER      0x01
#define  ENB_PACER      0x02

#define  EN_INT         0x80
#define  EN_DMA         0x04
#define  EOC            0x80

#define  DIFF_MODE      0x00
#define  SNGL_MODE      0x20
#define  NUM_PORTS      0x04
#define  IN_PORT        0x00
#define  OUT_PORT       0x01
#define  COMPLETE       0x00
#define  IN_PROG        0x01
#define  MODE_MASK      0xFF
#define  OPER_MASK      0xF00

#define  FALSE          0x00
#define  TRUE           0x01

   // I/O Port Address Map
#define  ADC_DATA       0x00     // A/D data register
#define  START_ADC      0x00     // writing to this addr. will initiate a conv
#define  RANGE_CTL      0x01     // A/D range/gain register
#define  ADC_MUX_CTL    0x02     // A/D multiplexer scan register
#define  DAC0           0x04     // D/A converter 0
#define  DAC1           0x06     // D/A converter 1
#define  ADC_STATUS     0x08     // A/D status register
#define  CLEAR_IRQ      0x08     // write any val to this addr to clear IRQ
#define  ADC_CTL        0x09     // A/D control register
#define  PIT_ENABLE     0x0A     // counter/timer enable/disable register
#define  PIT_CNT_0      0x0C     // 8254 counter/timer 0 load/status register
#define  PIT_CNT_1      0x0D     // 8254 counter/timer 1 load/status register 
#define  PIT_CNT_2      0x0E     // 8254 counter/timer 2 load/status register
#define  PIT_CTL        0x0F     // 8254 counter/timer control register
#define  P0_ADDR        0x03     // digital input/output port 0
#define  P1_ADDR        0x0B     // digital input/output port 1

   // defines for das_set_mux function
#define  SET_ALL_CHNLS  0x00
#define  SET_SNGL_CHNL  0x01

   // request types
#define  INT_OP         0x01
#define  DMA_OP         0x02
#define  DMA_POLL_OP    0x06
#define  NUM_AD_RANGES  0x09

   // defines for instrument amplifier gain/ranges (A/D)
                                    // input range    gain  uni/bipolar
#define  _UN_X1          4          // 0 to +10V      x1    unipolar 
#define  _UN_X2          5          // 0 to +5V       x2    unipolar
#define  _UN_X5          6          // 0 to +2V       x5    unipolar 
#define  _UN_X10         7          // 0 to 1V        x10   unipolar
#define  _BI_X_5         8          // +-10V          x0.5  bipolar
#define  _BI_X1          0          // +-5V           x1    bipolar
#define  _BI_X2          1          // +-2.5V         x2    bipolar
#define  _BI_X5          2          // +-1V           x5    bipolar
#define  _BI_X10         3          // +-0.5V         x10   bipolar

#define  CFG_FILE          "cfg_default.pcl"
#define  CFG_FILE_PATH     "/usr/lib/das/config/"
