// Metrabyte DAS-20 adatper
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// das_ad.c module contains routines to:
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//    perform a A/D conv via sw        - das_ad_sync()
//    perform 'N' A/D conv via
//       interrupts or dma             - das_ad_enqueue()
//    remove an entry from the finish
//       queue                         - das_ad_dequeue()
//    perform a blocking read on a
//       A/D request                   - das_ad_read()
//    perform a non-blocking read on a
//       A/D request                   - das_ad_cread()
//    terminate conversions on a chnl  - das_ad_term()
//    set the scan sequence for conv   - das_ad_set_scan()
//    return the range/gain on a chnl  - das_ad_get_range()
//    set the range/gain on a chnl     - das_ad_set_range()
//    set the trigger source on a chnl - das_ad_set_trig()
//    return the trig source on a chnl - das_ad_get_trig()
//    set the conv rate on a chnl      - das_ad_set_sample()
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#include <met_20.h>
#include <das.h>
#include <conio.h>
#include <das_struct.h>
#include <das_dma.h>
#include <das_extern.h>
#include <das_tmr.h>
#include <das_inline.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/kernel.h>
#include <sys/seginfo.h>

struct das_rng das_ad_rng[NUM_AD_RANGES] = {
          {         0, 10000000 },        //    0V - 10V
          { -10000000, 10000000 },        //  -10V - 10V
          {         0, 10000000 },        //    0V - 10V
          {  -5000000,  5000000 },        //   -5V - 5V
          {         0,  1000000 },        //    0V - 1V
          {   -500000,   500000 },        // -0.5V - 0.5V
          {         0,   100000 },        //    0V - 100mV
          {    -50000,    50000  } };     // -50mV - 50mV
struct   das_scan dscan;                  // scan queue

// Function das_ad_sync performs a single analog to digital conversion.
// Note the buffer must be as large as the A/D converter resolution.
// ie if the resolution is 12 bit, the buffer must be 2 bytes.
int das_ad_sync(int fd, int chnl, char *buf)
{
   if(das_chk_init_fd(fd) != EOK)
      return(-1);

   // Is this a valid channel #
   if(chnl > ((docb.info.chnl_cnfg == DAS_SINGLE_ENDED) ? MAX_SNGL_CHNL : MAX_DIF_CHNL))
   {
      errno = ECHRNG;
      return(-1);
   }

   // If requests are already enqueued, return busy
   if(docb.ad[0].r_hd)
   {
      errno = EBUSY;
      return(-1);
   }
   das_set_mux(SET_SNGL_CHNL, chnl);   // set the multiplexer
   outp(docb.info.port+START_ADC, 0);  // start a conversion
   delay_8X(1000);                     // wait for the conversion to complete
                                       // normally you would poll the chip for
                                       // EOC, but since we would like to be
                                       // able to perform A/D while D/A is
                                       // going on we just delay for a while.
                                       // Note:  polling to fast won't let the
                                       // amplifiers settle.
   *(das_t *)buf = inpw(docb.info.port + ADC_DATA);
                                       // replace the first entry in the queue
   das_set_mux(SET_SNGL_CHNL, dscan.scan[0]);
   return(EOK);
}

// Function das_ad_enqueue enqueues an A/D request.
int das_ad_enqueue(int fd, int chnl, das_buf_t *buf, int nbytes, int mode, pid_t proxy)
{
   switch(mode & OPER_MASK)                  // check for a valid mode
   {     
      case DAS_INT:
      case DAS_DMA:
         return(das_ad_request(fd, chnl, buf, nbytes, mode, proxy));

      case DAS_DMA_SYNC:
         if(das_ad_request(fd, chnl, buf, nbytes, mode, proxy) !=EOK)
            return(-1);
         return(das_ad_dma_sync());          // go poll DMA controller

      default:
         errno = EINVAL;
         return(-1);
   }
}

// Function das_ad_dma_sync polls the DMA controller for terminal count.
int das_ad_dma_sync()
{
                                          // poll the dma controller
   while(read_dma(0, docb.info.dma_chnl) != 0xFFFF)
      ;
   outp(docb.info.port + INT_CTL_REG, INT_DISABLE);   // disable INT
   outp(docb.info.port + ADC_CTL, DMA_UNACTIVE);      // disable DMA
   outp(docb.info.port + DMA_CTL_REG, DMA_DISABLE);   // disable DMA
   das_ct_stop(_das_fd, 4);
   das_ct_stop(_das_fd, 5);

      //  unlink from request queue and link to finish queue
   das_link(das_unlink(0, &docb.ad[0].r_hd, &docb.ad[0].r_tl),
          &docb.ad[0].f_hd, &docb.ad[0].f_tl);
   docb.ad[0].n_rent--;
   docb.ad[0].n_fent++;
   docb.ad[0].stat = COMPLETE;
   Trigger(docb.ad[0].f_tl->proxy);
   return(0);
}

// Function das_ad_request places a conversion request into the request
// queue associated with 'chnl'.
int das_ad_request(int fd, int chnl, das_buf_t *buf, int nbytes, int mode, pid_t proxy)
{
   struct das_queue *ptr;

   if(das_chk_init_fd(fd) != EOK)
      return(-1);

   if(chnl != 0)                             // only take channel zero for
   {                                         // adapters with multiplexed A/D
      errno = ECHRNG;
      return(-1);
   }

   // If there are request in the queue and mode is DMA_SYNC return busy
   if(docb.ad[0].r_hd && (mode & DAS_DMA_SYNC))
   {
      errno = EBUSY;
      return(-1);
   }

   if(docb.da[0].r_hd)                       // adapter doesn't support A/D
   {                                         // and D/A at the same time
      errno = EBUSY;
      return(-1);
   }

   // limit number of entries in the queues
   if((docb.ad[0].n_rent + docb.ad[0].n_fent) == MAX_QUEUE_ENT)
   {
      errno = EBUSY;
      return(-1);
   }

   // If mode is INT and sample rate is greater than 10kHz return EINVAL
   if((mode & DAS_INT) && (docb.ad[0].s_rate.div1 * docb.ad[0].s_rate.div2)
          < INT_TICKS )
   {
      errno = EINVAL;
      return(-1);
   }

   // check for a valid number of conversions
   if(!nbytes || nbytes > MAX_CONV)
   {
      errno = EINVAL;
      return(-1);
   }

   // check for a valid mode
   if(((mode & MODE_MASK) != DAS_SNGL_CYCLE) && ((mode & MODE_MASK) != 
         DAS_RECYCLE) && ((mode & MODE_MASK) != DAS_NOTIFY))
   {
      errno = EINVAL;
      return(-1);
   }

   if(docb.ad[chnl].stat == DAS_RECYCLE)     // no sense queueing any more if
   {                                         // a recycle is in the queue
      errno = EBUSY;
      return(-1);
   }

   if(buf->bufp == NULL)                     // valid buffer?
   {
      errno = EINVAL;
      return(-1);
   }

   // create a request entry and fill it in
   if((ptr = das_create_entry()) == (struct das_queue *)NULL)
      return(-1);
   ptr->mode = mode & 0xFF;                  // mode bits 0 - 4
   ptr->rtype = mode & 0xF00;                // request type bits 8 - 11
   ptr->proxy = proxy;                       // proxy to trigger on completion 
   ptr->bytes = ptr->cnt = nbytes;           // number of bytes requested
   ptr->dma = buf;                           // structure from das_alloc
   ptr->cp = (das_t *)buf->bufp;             // pointer to buffer
   docb.ad[chnl].n_rent++;                   // increment num request entries
   docb.ad[chnl].stat = mode;                // set status to mode

   // link request to tail of request queue, if queue is emptry initiate
   // the request
   if(das_link(ptr, &docb.ad[chnl].r_hd, &docb.ad[chnl].r_tl))
      das_ad_cmd(chnl);
   return(EOK);
}

// Function das_ad_read() performs a blocking read on a A/D request.  A check
// is made to make sure they don't block if they haven't initiated any
// conversions.  The proxy for the request is determined and the Receive
// function is called.  If DAS_REUSE is specified the request is reenqueued.
// The acquired data is copied into the specified buffer and the number of
// bytes is returned.
// Note this function will return EINVAL in a proxy hasn't been specified
// in the das_ad_enqueue function.  This function should not be called by
// servers.
int das_ad_read(int fd, char *buf, int nbytes, int chnl, int reuse)
{
   int psw, rtype;
   pid_t proxy;
   
   if(das_chk_init_fd(fd) != EOK)            // library initialized?
      return(-1);

   if(chnl != 0)                             // only take channel zero for
   {                                         // adapters with multiplexed A/D
      errno = ECHRNG;
      return(-1);
   }

   if(buf == NULL)                           // valid pointer?
   {
      errno = EINVAL;
      return(-1);
   }

   // Make sure they don't block if they haven't initiated any conversions
   psw = disable();
   if(docb.ad[chnl].f_hd == NULL && docb.ad[chnl].r_hd == NULL)
   {
      errno = EDEADLK;
      restore(psw);
      return(-1);
   }

   // proxies aren't triggered when we are in recycle mode
   if(docb.ad[chnl].stat == DAS_RECYCLE && docb.ad[chnl].f_hd == NULL)
   {
      errno = EDEADLK;
      restore(psw);
      return(-1);
   }
   if(!docb.ad[chnl].f_hd)                   // get the correct proxy to receive on
      proxy = docb.ad[chnl].r_hd->proxy;
   else
      proxy = docb.ad[chnl].f_hd->proxy;
   restore(psw);

   if(!proxy)                             // no proxy to block on
   {
      errno = EINVAL;
      return(-1);
   }

   while(!docb.ad[0].f_hd)                // wait for an entry in the queue
   {
      if(Receive(proxy, 0, 0 ) == -1)        // when an entry is put in the
         return(-1);                         // queue we will always get
   }                                         // get a proxy to unblock us.

   // If they ask for more 'bytes' than was in the actual request, give them
   // the number of bytes in the request.  Otherwise give them what they
   // asked for.
   if(nbytes > docb.ad[chnl].f_hd->bytes)
      nbytes = docb.ad[chnl].f_hd->bytes;

   // copy the acquired data into the buffer passed in.
   das_memcpy16(buf, docb.ad[chnl].f_hd->dma->bufp, nbytes);

   // Make sure a D/A request isn't in progress they want to reuse the buffer.
   if(reuse && docb.da[0].r_hd)
   {
      das_unlink(1, &docb.ad[0].f_hd, &docb.ad[0].f_tl);
      docb.ad[0].n_fent--;
      errno = EBUSY;
      return(-1);
   }

   if(reuse)                              // re-enqueue the request?
   {
      rtype = docb.ad[0].f_hd->rtype;
      docb.ad[0].f_hd->cnt = docb.ad[0].f_hd->bytes;
      docb.ad[0].f_hd->cp = (das_t *)docb.ad[0].f_hd->dma->bufp;
      docb.ad[chnl].n_rent++; 
      docb.ad[chnl].stat = docb.ad[0].f_hd->mode;

      //  unlink from finish queue and link to request queue
      if(das_link(das_unlink(0, &docb.ad[0].f_hd, &docb.ad[0].f_tl),
               &docb.ad[0].r_hd, &docb.ad[0].r_tl))
         das_ad_cmd(chnl);
      if(rtype == DAS_DMA_SYNC)
         das_ad_dma_sync();
   }
   else              // unlink request from finish queue and free its memory
      das_unlink(1, &docb.ad[chnl].f_hd, &docb.ad[chnl].f_tl);
   docb.ad[0].n_fent--;             // decrement num entries in finish queue
   return(nbytes);                  // return the number of conversions
}

// Function das_ad_cread() performs a non-blocking read on a A/D request.
// The 'chnl' finish queue is checked for an entry.  If no entries exist in
// the queue, a 0 is returned.  Otherwise the entry it is unlinked from
// it and the number of conversions is returned.  If DAS_REUSE is specified
// the request is re-enqueued.
int das_ad_cread(int fd, char *buf, int nbytes, int chnl, int reuse)
{
   int rtype;
   
   if(das_chk_init_fd(fd) != EOK)            // library initialized?
      return(-1);

   if(chnl != 0)                             // only take channel zero for
   {                                         // adapters with multiplexed A/D
      errno = ECHRNG;
      return(-1);
   }

   if(buf == NULL)                           // valid buffer?
   {
      errno = EINVAL;
      return(-1);
   }

   if(!docb.ad[chnl].f_hd)                   // anything in the finish queue?
      return(0);

   // If they ask for more 'bytes' than was in the actual request, give them
   // the number of bytes in the request.  Otherwise give them what they
   // asked for.
   if(nbytes > docb.ad[chnl].f_hd->bytes)
      nbytes = docb.ad[0].f_hd->bytes;

   // copy the acquired data into the buffer passed in.
   das_memcpy16(buf, docb.ad[0].f_hd->dma->bufp, nbytes);

   // Make sure a D/A request isn't in progress they want to reuse the buffer.
   if(reuse && docb.da[0].r_hd)
   {
      das_unlink(1, &docb.da[0].f_hd, &docb.da[0].f_tl);
      docb.da[0].n_fent--;
      errno = EBUSY;
      return(-1);
   }
   if(reuse)                              // re-enqueue the request?
   {
      rtype = docb.ad[0].f_hd->rtype;
      docb.ad[0].f_hd->cnt = docb.ad[0].f_hd->bytes;
      docb.ad[0].f_hd->cp = (das_t *)docb.ad[0].f_hd->dma->bufp;
      docb.ad[chnl].n_rent++; 
      docb.ad[chnl].stat = docb.ad[0].f_hd->mode;

      //  unlink from finish queue and link to request queue
      if(das_link(das_unlink(0, &docb.ad[0].f_hd, &docb.ad[0].f_tl),
               &docb.ad[0].r_hd, &docb.ad[0].r_tl))
         das_ad_cmd(chnl);
      if(rtype == DAS_DMA_SYNC)
         das_ad_dma_sync();
   }
   else              // unlink request from finish queue and free its memory
      das_unlink(1, &docb.ad[chnl].f_hd, &docb.ad[chnl].f_tl);
   docb.ad[0].n_fent--;
   return(nbytes);                              // return the number of conversions
}

// Function das_ad_dequeue removes an entry from the finish queue and returns
// a pointer to it.  If there are no entries in the finish queue it returns
// with an error.
das_buf_t * das_ad_dequeue(int fd, int chnl)
{
   das_buf_t *tmp;

   if(das_chk_init_fd(fd) != EOK)            // libary initialized?
      return(NULL);

   if(chnl != 0)                             // only take channel zero for
   {                                         // adapters with multiplexed A/D
      errno = ECHRNG;
      return(NULL);
   }

   if(!docb.ad[0].f_hd)                      // entries in finish queue?
   {
      errno = EACCES;
      return(NULL);
   }

   tmp = docb.ad[0].f_hd->dma;               // pointer to das_buf_t structure
      // unlink entry from finish queue and free the memory
   das_unlink(1, &docb.ad[chnl].f_hd, &docb.ad[chnl].f_tl);
   docb.ad[0].n_fent--;             // decrement num entries in finish queue
   return(tmp);
}

// Function das_ad_term() stops conversions on 'chnl'.  The entries in the
// 'ready' and 'finish' queues are removed.
int das_ad_term(int fd, int chnl)
{
   struct das_queue *r_hd, *f_hd, *r_tl, *f_tl;
   int psw;
   
   if(das_chk_init_fd(fd) != EOK)            // library initialized?
      return(-1);

   if(chnl != 0)                             // only take channel zero for
   {                                         // adapters with multiplexed A/D
      errno = ECHRNG;
      return(-1);
   }

   das_ct_stop(_das_fd, 4);                  // stop counters/timers
   das_ct_stop(_das_fd, 5);

   psw = disable();
   r_hd = docb.ad[chnl].r_hd;
   f_hd = docb.ad[chnl].f_hd;
   r_tl = docb.ad[chnl].r_tl;
   f_tl = docb.ad[chnl].f_tl;
   docb.ad[chnl].r_hd = docb.ad[chnl].r_tl = NULL;
   docb.ad[chnl].f_hd = docb.ad[chnl].f_tl = NULL;
   restore(psw);

// terminate the current interrupt or dma conversions
   if(r_hd)
   {
      docb.ad[chnl].stat = COMPLETE;
      outp(docb.info.port + INT_CTL_REG, INT_DISABLE);   // disable INT
      outp(docb.info.port + ADC_CTL, DMA_UNACTIVE);      // disable DMA
      outp(docb.info.port + DMA_CTL_REG, DMA_DISABLE);   // disable DMA
      if(r_hd->rtype == DAS_DMA)
         clear_dma(0, docb.info.dma_chnl);
   }
   docb.ad[chnl].n_fent = 0;           // reset num request and finish entries
   docb.ad[chnl].n_rent = 0;
   
   // clean up the queues
   while(r_hd)
      das_unlink(1, &r_hd, &r_tl);
   while(f_hd)
   {
      Creceive(f_hd->proxy, 0, 0);
      das_unlink(1, &f_hd, &f_tl);
   }
   return(EOK);
}

// Function das_set_mux() programs the gain/range queue ram and scan sequence.
int das_set_mux(int mode, int chnl)
{
   int cnt = 0, adc_reg, psw;
                        // set the queue for read/write access (1st entry)
   psw = disable();
   adc_reg = inp(docb.info.port + ADC_CTL);
   outp(docb.info.port + ADC_CTL, QUEUE_RW);
   inp(docb.info.port + ADC_CTL);            // reset the queue pointer
   if(mode == SET_SNGL_CHNL)
   {
      outp(docb.info.port + QUEUE_LOAD, (chnl << 4) | (docb.ad[chnl].rng << 1));
      outp(docb.info.port + ADC_CTL, QUEUE_STATIC | (adc_reg & 0x20));
      inp(docb.info.port + ADC_CTL);         // reset the queue pointer
      restore(psw);
      return(EOK);
   }
   restore(psw);
   for(cnt=0;cnt<=dscan.last; cnt++)
      outp(docb.info.port + QUEUE_LOAD, (dscan.scan[cnt] << 4) | (docb.ad[cnt].rng << 1));
   outp(docb.info.port + QUEUE_LOAD, (dscan.scan[cnt] << 4) | (docb.ad[cnt].rng << 1) | END_OF_QUEUE);
   return(EOK);
}

// Function das_ad_set_scan() sets the scan queue.  The scan sequence is checked
// for validity and the hardware range/gain queuing RAM is programmed.
int das_ad_set_scan(int fd, char *scanlist, int n_entries)
{
   int cnt, chnl;
   
   if(das_chk_init_fd(fd) != EOK)         // libary initialized?
      return(-1);

   if(docb.ad[0].r_hd)                    // don't allow scan sequence to
   {                                      // change until queue is empty
      errno = EBUSY;
      return(-1);
   }
                                          // check for valid number of entries
   if(n_entries <= 0 || n_entries > QUEUE_SIZE)
   {
      errno = EINVAL;
      return(-1);
   }
   chnl = (docb.info.chnl_cnfg == DAS_SINGLE_ENDED) ? MAX_SNGL_CHNL : MAX_DIF_CHNL;
   for(cnt=0; cnt<n_entries; cnt++)             // verify scan list
      if(scanlist[cnt] > chnl)
      {
         errno = ECHRNG;
         return(-1);
      }
   memcpy(&dscan.scan[0], scanlist, n_entries); // make a copy of the scan list
   dscan.last = n_entries-1;                    // set last entry in list
   das_set_mux(SET_ALL_CHNLS, 0);
   return(EOK);
}

// Function das_ad_get_range() returns the range/gain on 'chnl'.
int das_ad_get_range(int fd, int chnl, long *rmin, long *rmax)
{
   if(das_chk_init_fd(fd) != EOK)
      return(-1);
                                          // check for a valid channel number
   if(chnl > ((docb.info.chnl_cnfg == DAS_SINGLE_ENDED) ? MAX_SNGL_CHNL : MAX_DIF_CHNL))
   {
      errno = ECHRNG;
      return(-1);
   }

   *rmin = das_ad_rng[docb.ad[chnl].rng].min;
   *rmax = das_ad_rng[docb.ad[chnl].rng].max;
   return(EOK);
}

// Function das_ad_set_range() sets the range/gain on 'chnl'.  Returns zero on
// success -1 on error.
int das_ad_set_range(int fd, int chnl, long rmin, long rmax)
{
   int cnt;
   
   if(das_chk_init_fd(fd) != EOK)         // library initialized?
      return(-1);

   if(docb.ad[0].r_hd)                    // don't allow range change
   {                                      // until queue is empty
      errno = EBUSY;
      return(-1);
   }
                                          // check for a valid channel number
   if(chnl > ((docb.info.chnl_cnfg == DAS_SINGLE_ENDED) ? MAX_SNGL_CHNL : MAX_DIF_CHNL))
   {
      errno = ECHRNG;
      return(-1);
   }

   for(cnt=0;cnt<NUM_AD_RANGES; cnt++)    // verify range
      if(das_ad_rng[cnt].min == rmin && das_ad_rng[cnt].max == rmax)
         break;
   
   if(cnt == NUM_AD_RANGES)               // invalid range
   {
      errno = EINVAL;
      return(-1);
   }
   docb.ad[chnl].rng = cnt;
   das_set_mux(SET_ALL_CHNLS, 0);
   return(EOK);
}

// Function das_ad_set_trig() sets the trigger source on 'chnl'.
int das_ad_set_trig(int fd, int chnl, int trig_src)
{
   unsigned int trig;
   
   if(das_chk_init_fd(fd) != EOK)         // libary initialized?
      return(-1);

   if(docb.ad[0].r_hd)                    // don't allow trigger change
   {                                      // until queue is empty
      errno = EBUSY;
      return(-1);
   }

   if(chnl != 0)                 // Just allow channel zero trigger to be set
   {
      errno = ECHRNG;
      return(-1);
   }
   switch(trig_src)              // check for a valid trigger source
   {
      case DAS_SW_TRIG:
         trig = _SW_TRIG;
         break;
      case DAS_SW_TRIG | DAS_ENB_GATE | DAS_A_LOW:
         trig = _SW_TRIG | _ENB_GATE;
         break;
      case DAS_SW_TRIG | DAS_ENB_GATE | DAS_A_HIGH:
         trig = _SW_TRIG | _ENB_GATE | _A_HIGH;
         break;
      case DAS_PC_TRIG:             // pacer trigger uses counter 5
         trig = _HW_TRIG;
         break;
      case DAS_PC_TRIG | DAS_ENB_GATE | DAS_A_LOW:
         trig = _HW_TRIG | _ENB_GATE;
         break;
      case DAS_PC_TRIG | DAS_ENB_GATE | DAS_A_HIGH:
         trig = _HW_TRIG | _ENB_GATE | _A_HIGH;
         break;
      case DAS_EX_TRIG:
         trig = _HW_TRIG | _EX_TRIG;
         break;
      case DAS_EX_TRIG | DAS_ENB_GATE | DAS_A_LOW:
         trig = _HW_TRIG | _EX_TRIG | _ENB_GATE;
         break;
      case DAS_EX_TRIG | DAS_ENB_GATE | DAS_A_HIGH:
         trig = _HW_TRIG | _EX_TRIG | _ENB_GATE | _A_HIGH;
         break;
      default:
         errno = EINVAL;
         return(-1);
   }
   // store adapter specific trigger for ADC control register in upper 8 bits
   docb.ad[chnl].trig_src = (trig << 8) | trig_src;
   return(EOK);
}

// Function das_ad_get_range() returns the trigger source on 'chnl'.
int das_ad_get_trig(int fd, int chnl, int *trig_src)
{
   if(das_chk_init_fd(fd) != EOK)         // libary initialized?
      return(-1);

   if(chnl != 0)                 // Just allow channel zero trigger to be set
   {
      errno = ECHRNG;
      return(-1);
   }
   *trig_src = docb.ad[chnl].trig_src & 0xFF;
   return(EOK);
}

// Function das_ad_set_sample() sets the conversion rate on 'chnl'.
int das_ad_set_sample(int fd, int chnl, unsigned long ticks)
{
   if(das_chk_init_fd(fd) != EOK)         // libary initialized?
      return(-1);

   if(chnl != 0)                 // Just allow channel rate to be set
   {
      errno = ECHRNG;
      return(-1);
   }

   if(ticks < MIN_TICKS || ticks > MAX_TICKS)   // valid sample rate
   {
      errno = EINVAL;
      return(-1);
   }
      
   // convert ticks parameter into two divisors that can be sent to counters
   das_convert_rate(ticks, &docb.ad[chnl].s_rate);
   docb.cntrs.ctr[4-1].ld = docb.ad[chnl].s_rate.div2;
   docb.cntrs.ctr[5-1].ld = docb.ad[chnl].s_rate.div1;
   if(docb.ad[chnl].r_hd && (docb.ad[chnl].trig_src & DAS_PC_TRIG))
   {
      das_ct_write(_das_fd, 4, &docb.cntrs.ctr[4-1]);
      das_ct_write(_das_fd, 5, &docb.cntrs.ctr[5-1]);
   }
   return(EOK);
}

// Function das_convert_rate() converts a sample rate in ticks into two divisor
// values that can be written to the counters.
int das_convert_rate(unsigned long ticks, struct das_rate *sptr)
{
   sptr->div1 = 0;
   sptr->div2 = 1;
   
   do
   {
      sptr->div2*=2;
      sptr->div1 = ticks / sptr->div2;
   }while(sptr->div1 > 0xFFFF);
   return(EOK);
}

// Function das_ad_rqueue returns the number of requests in the ready queue
// for a 'chnl'.
int das_ad_rqueue(int fd, int chnl)
{
   if(das_chk_init_fd(fd) != EOK)         // libary initialized?
      return(-1);

   // valid channel?
   if(chnl > ((docb.info.chnl_cnfg == DAS_SINGLE_ENDED) ? MAX_SNGL_CHNL : MAX_DIF_CHNL))
   {
      errno = ECHRNG;
      return(-1);
   }
   return(docb.ad[0].n_rent);
}

// Function das_ad_fqueue returns the number of requests in the ready queue
// for a 'chnl'.
int das_ad_fqueue(int fd, int chnl)
{
   if(das_chk_init_fd(fd) != EOK)         // libary initialized?
      return(-1);
   
   // valid channel?
   if(chnl > ((docb.info.chnl_cnfg == DAS_SINGLE_ENDED) ? MAX_SNGL_CHNL : MAX_DIF_CHNL))
   {
      errno = ECHRNG;
      return(-1);
   }
   return(docb.ad[0].n_fent);
}
