// Metrabyte DAS-20 Adapter
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// das_int.c module contains routines to:
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//    interrupt handler                - das_handler()
//    program adapter for ad conver.   - das_ad_cmd()
//    program adapter for da conver.   - das_da_cmd()
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// If the memory model isn't small, compile with the -zu option

#include <conio.h>
#include <stdio.h>
#include <met_20.h>
#include <das_extern.h>
#include <das_struct.h>
#include <das.h>
#include <das_dma.h>
#include <das_inline.h>

#ifdef _DAS_MAN
volatile das_t das_res_buf[16];
extern int das_res_int(void);
#endif

#pragma off (check_stack)
// Function das_handler() processes interrupts from the adapter.  Possible
// interrupt sources are:  DMA terminal count
//                         End of Conversion
//                         Timer #2 Output
//                         End of Queue
pid_t far das_handler()
{
   register struct   das_chnl    *p;
   int   isr, dsr;

   if(docb.ad[0].r_hd == NULL && docb.da[0].r_hd == NULL)
      return(0);
                              // read the interrupt and DMA status registers
   isr = inp(docb.info.port + INT_CTL_REG);
   dsr = inp(docb.info.port + DMA_CTL_REG) & XFR_MASK;

   switch(isr & INT_MASK)
   {
      case EOC_INT:                 // ADC End of Conversion interrupt
         p = &docb.ad[0];
         if((p->r_hd->mode & DAS_RECYCLE) && !p->r_hd->cnt)
         {
            p->r_hd->cp = (das_t *)p->r_hd->dma->bufp;
            p->r_hd->cnt = p->r_hd->bytes;
         }
         *p->r_hd->cp++ = inpw(docb.info.port + ADC_DATA);
         p->r_hd->cnt -= sizeof(das_t);
         if(!p->r_hd->cnt && (!(p->r_hd->mode & DAS_RECYCLE)))
         {
               //  unlink from request queue and link to finish queue
            das_link(das_unlink(0, &p->r_hd, &p->r_tl), &p->f_hd, &p->f_tl);
            p->n_rent--;            // decrement num entries in request queue
            p->n_fent++;            // increment num entries in finish queue
            outp(docb.info.port + INT_CTL_REG, INT_DISABLE);   // disable INT
            if(p->r_hd)                         // more requests to process?
               das_ad_cmd(0);                   // process next entry
            else
            {
               docb.ad[0].stat = COMPLETE;
               das_ctr_cmd(4, _DARM_CTR);
               das_ctr_cmd(5, _DARM_CTR);
            }
            return(docb.ad[0].f_tl->proxy);
         }
         break;

      case TM2_INT:                 // Timer #2 Output interrupt (D/A)
         p = &docb.da[0];
      
         if((p->r_hd->mode & DAS_RECYCLE) && !p->r_hd->cnt)
         {
            p->r_hd->cp = (das_t *)p->r_hd->dma->bufp;
            p->r_hd->cnt = p->r_hd->bytes;
         }
         if(p->r_hd->mode & DAS_DUAL_CHNL)
         {
            outpw(docb.info.port + docb.da[0].addr, *p->r_hd->cp++);
            outpw(docb.info.port + docb.da[1].addr, *p->r_hd->cp++);
            p->r_hd->cnt -= sizeof(das_t);
         }
         else
            outpw(docb.info.port + docb.da[(p->r_hd->mode&0xF00)>>8].addr, *p->r_hd->cp++);
         p->r_hd->cnt -= sizeof(das_t);

         if(!p->r_hd->cnt && (!(p->r_hd->mode & DAS_RECYCLE)))
         {
            das_ctr_cmd(2, _DARM_CTR);
            das_ctr_cmd(1, _DARM_CTR);
            outp(docb.info.port + INT_CTL_REG, INT_DISABLE);   // disable INT
               //  unlink from request queue and link to finish queue
            das_link(das_unlink(0, &p->r_hd, &p->r_tl), &p->f_hd, &p->f_tl);
            p->n_rent--;
            p->n_fent++;
            if(p->r_hd)
               das_da_cmd(0);
            else
               docb.da[0].stat = COMPLETE;
            return(docb.da[0].f_tl->proxy);
         }
         break;

      case EQU_INT:                 // End of Instruction Queue interrupt
         break;

      case DMA_INT:                 // DMA Terminal Count interrupt
         if(dsr)
            p = &docb.da[0];
         else
         {
            p = &docb.ad[0];
#ifdef _DAS_MAN
// This section of code is required for the Acquisition Manager.  We use the
// DMA terminal count as a time base.  This time base is scaled to produce
// the conversion rate specified by the das_ad_set_sample and das_da_set_sample
// functions.  When the DMA transfer completes the DMAC is re-programmed and
// the das_res_int function is called.  The das_res_int function increments a
// wall clock.  The wall clock is then compared to the first entry in the
// tick queue.  Equal values indicate that a conversion should be performed.
// A proxy is triggered to signify this event.  This event is then processed
// by the manager at process time.
            if(p->r_hd->mode & DAS_NOTIFY)      // used by resource manager
            {     
               // copy data from active sample buffer to buffer for manager
               das_memcpy16(das_res_buf, p->r_hd->dma->bufp, p->r_hd->bytes);
               das_ctr_cmd(4, _DARM_CTR);
               das_ctr_cmd(5, _DARM_CTR);
               das_ad_cmd(0);                   // program the DMA controller
               if (das_res_int())
                  return(docb.ad[0].r_hd->proxy);
               return(0);
            }
#endif
         }
            //  unlink from request queue and link to finish queue
         das_link(das_unlink(0, &p->r_hd, &p->r_tl), &p->f_hd, &p->f_tl);
         p->n_rent--;
         p->n_fent++;
         // disable adapters DMA and interrupts
         outp(docb.info.port + INT_CTL_REG, INT_DISABLE);
         outp(docb.info.port + ADC_CTL, DMA_UNACTIVE);
         outp(docb.info.port + DMA_CTL_REG, DMA_DISABLE);
         if(p->r_hd)
         {
            if(dsr)
            {
               das_ctr_cmd(2, _DARM_CTR);
               das_ctr_cmd(1, _DARM_CTR);
               das_da_cmd(0);
            }
            else
            {
               das_ctr_cmd(4, _DARM_CTR);
               das_ctr_cmd(5, _DARM_CTR);
               das_ad_cmd(0);
            }
         }
         else
         {
            if(dsr)
            {
               docb.da[0].stat = COMPLETE;
               das_ctr_cmd(2, _DARM_CTR);
               das_ctr_cmd(1, _DARM_CTR);
            }
            else
            {
               docb.ad[0].stat = COMPLETE;
               das_ctr_cmd(4, _DARM_CTR);
               das_ctr_cmd(5, _DARM_CTR);
            }
         }
         return(dsr ? docb.da[0].f_tl->proxy : docb.ad[0].f_tl->proxy);
      break;
   }
   outp(docb.info.port + INT_CTL_REG, isr);     // clear the interrupt
   return(0);
}
#pragma on (check_stack)

#pragma off (check_stack)
// Function das_ad_cmd() programs the adapter using the information stored
// in the queue request entry
void das_ad_cmd(int chnl)
{
   int   dma_ctl, int_ctl, adc_ctl;

   // disable adapters DMA and interrupts
   outp(docb.info.port + INT_CTL_REG, INT_DISABLE);
   outp(docb.info.port + ADC_CTL, DMA_UNACTIVE);
   outp(docb.info.port + DMA_CTL_REG, DMA_DISABLE);

   int_ctl = (docb.info.irq << 4);     // shift irq into proper bit location
   adc_ctl = (docb.ad[0].trig_src >> 8) | QUEUE_WRAP;

   inp(docb.info.port+ADC_CTL);        // reset the queue pointer

   das_ctr_cmd(4, _DARM_CTR);          // stop the 9513 counters
   das_ctr_cmd(5, _DARM_CTR);

   if(adc_ctl & _HW_TRIG)              // are we using the 9513 counters
   {                                   // setup counters/timers
      das_ct_mode(_das_fd, 4, &docb.cntrs.ctr[3].md, NULL); // cntr 4
      das_ct_mode(_das_fd, 5, &docb.cntrs.ctr[4].md, NULL); // cntr 5
      das_ct_write(_das_fd, 4, &docb.cntrs.ctr[3]);         // cntr 4
      das_ct_write(_das_fd, 5, &docb.cntrs.ctr[4]);         // cntr 5
   }
   
   if(docb.ad[chnl].r_hd->rtype & DAS_DMA || docb.ad[chnl].r_hd->rtype & DAS_DMA_SYNC)
   {
      // set the adapter dma control register
      dma_ctl = (docb.info.dma_chnl == MAX_DMA_CHNL) ? DMA_CH3 : DMA_CH1;
      outp(docb.info.port + DMA_CTL_REG, dma_ctl | DMA_ENABLE | ADC_DMA);

      // if it is in RECYCLE or DMA_SYNC mode don't enable the interrupts
      if(!(docb.ad[chnl].r_hd->mode & DAS_RECYCLE) &&
                                    docb.ad[chnl].r_hd->rtype != DAS_DMA_SYNC)
         int_ctl |= INT_ENABLE;           // enable adapter interrupts
      int_ctl |= DMA_INT;                 // interrupt on DMA terminal count

      setup_dma((docb.ad[chnl].r_hd->mode & DAS_RECYCLE) ? DMA_AUTO : 0,
                  &docb.ad[chnl].r_hd->dma->dma, docb.ad[chnl].r_hd->bytes,
                  docb.info.dma_chnl);
      adc_ctl |= DMA_ACTIVE;              // enable dma xfrs on adapter
   }
   else                                   // must be an INTERRUPT driven xfer
      int_ctl |= INT_ENABLE;        
   
   // set the interrupt control register
   outp(docb.info.port + INT_CTL_REG, int_ctl);    
   // set the ADC control register
   outp(docb.info.port + ADC_CTL, adc_ctl);
}
#pragma on (check_stack)

// Function das_da_cmd() programs the adapter using the information stored
// in the queue request entry
#pragma off (check_stack)
void das_da_cmd(int chnl)
{
   int   dma_ctl, int_ctl, adc_ctl;

   outp(docb.info.port + INT_CTL_REG, INT_DISABLE);   // disable INT
   outp(docb.info.port + ADC_CTL, DMA_UNACTIVE);      // disable DMA
   outp(docb.info.port + DMA_CTL_REG, DMA_DISABLE);   // disable DMA

   int_ctl = (docb.info.irq << 4);        // put irq in proper bit location
   adc_ctl = docb.da[0].trig_src >> 8;    // trigger source in ADC cntl reg

   das_ctr_cmd(2, _DARM_CTR);             // stop counters
   das_ctr_cmd(1, _DARM_CTR);

   if(adc_ctl & _HW_TRIG)
   {
      das_ct_mode(_das_fd, 1, &docb.cntrs.ctr[0].md, NULL); // cntr 1
      das_ct_mode(_das_fd, 2, &docb.cntrs.ctr[1].md, NULL); // cntr 2
      das_ct_write(_das_fd, 1, &docb.cntrs.ctr[0]);         // cntr 1
      das_ct_write(_das_fd, 2, &docb.cntrs.ctr[1]);         // cntr 2
   }

   if(docb.da[chnl].r_hd->rtype & DAS_DMA || docb.da[chnl].r_hd->rtype & DAS_DMA_SYNC)
   {
      if(docb.da[chnl].r_hd->mode & DAS_DUAL_CHNL)    // dual issue xfer?
         dma_ctl = DAC1_2_DMA;
      else                       // channel number is bit 8 in mode
         dma_ctl = ((docb.da[chnl].r_hd->mode & 0x100) == 0) ? DAC0_DMA : DAC1_DMA;

      dma_ctl |= (docb.info.dma_chnl == MAX_DMA_CHNL) ? DMA_CH3 : DMA_CH1;
      outp(docb.info.port + DMA_CTL_REG, dma_ctl | DMA_ENABLE);

      // if it is in RECYCLE or DMA poll mode don't enable the interrupts
      if(!(docb.da[chnl].r_hd->mode & DAS_RECYCLE) && docb.da[chnl].r_hd->rtype != DAS_DMA_SYNC)
         int_ctl |= INT_ENABLE;        
      int_ctl |= DMA_INT;
      setup_dma(DMA_OUTPUT | ((docb.da[chnl].r_hd->mode & DAS_RECYCLE) ? DMA_AUTO : 0),
            &docb.da[chnl].r_hd->dma->dma, docb.da[chnl].r_hd->bytes, docb.info.dma_chnl);
      adc_ctl |= DMA_ACTIVE;
   }
   else                                // must be and INTERRUPT driven request
      int_ctl |= INT_ENABLE | TM2_INT;

   // set interrupt control register
   outp(docb.info.port + INT_CTL_REG, int_ctl);
   // set ADC control register
   outp(docb.info.port + ADC_CTL, adc_ctl);
}
#pragma on (check_stack)
