//*****************************************************************************
// Module das_dig_msg.c contains the messaging functions for the 'das' resource
// manager driver.  These functions send all digital I/O operation messages 
// to the resource manager though a file descriptor.
//*****************************************************************************
// The follwing is the list of functions in this module.
//
//    das_dig_out
//    das_dig_in
//*****************************************************************************
#include <errno.h>
#include <sys/kernel.h>
#include <das.h>
#include "das_struct_msg.h"

extern union das_msg    io_msg;

//*****************************************************************************
// das_digital_out - Send message to output data on digital port specified by 
//                   'port'.
//*****************************************************************************
int das_digital_out(int fd, int port, long bits, long mask, long *old_bits, long *new_bits)
{
                     // Fill in message structure.
   io_msg.das_dout.type = _IO_DAS;
   io_msg.das_dout.request = _DAS_DIG_OUT; 
   io_msg.das_dout.fd = fd; 
   io_msg.das_dout.port = port; 
   io_msg.das_dout.bits = bits;
   io_msg.das_dout.mask = mask;

   if(Sendfd(fd, &io_msg, &io_msg, sizeof(io_msg.das_dout), 
               sizeof(io_msg.das_dout_reply)) == -1)
      return(-1);
   if(io_msg.status != EOK)
   {
      errno = io_msg.status;
      return(-1);
   }
   if (old_bits)
      *old_bits = io_msg.das_dout_reply.old_bits;
   if (new_bits)
      *new_bits = io_msg.das_dout_reply.new_bits;
   return(EOK);
}

//*****************************************************************************
// das_digital_in -  Send message to input data on digital port specified by 
//                   'port'.
//*****************************************************************************
int das_digital_in(int fd, int port, long *bits)
{
                     // Fill in message structure.
   io_msg.das_din.type = _IO_DAS;
   io_msg.das_din.request = _DAS_DIG_IN; 
   io_msg.das_din.fd = fd; 
   io_msg.das_din.port = port; 

   if(Sendfd(fd, &io_msg, &io_msg, sizeof(io_msg.das_din), 
               sizeof(io_msg.das_din_reply)) == -1)
      return(-1);
   if(io_msg.status != EOK)
   {
      errno = io_msg.status;
      return(-1);
   }
   *bits = io_msg.das_din_reply.bits;  
   return(EOK);
}
