//*****************************************************************************
// das_mem_msg.c module contains routines to:
//*****************************************************************************
//    allocate memory buffers    - das_alloc()
//    deallocate memory buffers  - das_free_mem()
//*****************************************************************************

#include <errno.h>
#include <malloc.h>
#include <sys/kernel.h>
#include <das.h>
#include "das_struct_msg.h"

extern union das_msg    io_msg;
extern das_info das_query_info;

// Function das_alloc() allocates nbytes of memory and returns a far pointer
// to it.
das_buf_t * das_alloc(int fd, int nbufs, int nbytes, unsigned flags)
{
   int cnt;
   das_buf_t *dptr;
      
   if(nbytes > 0xFFFF)
   {
      errno = EINVAL;
      return(NULL);
   }

   if(nbytes == 0 || nbufs == 0)
   {
      errno = EINVAL;
      return(NULL);
   }
                     // Fill in message structure.
   io_msg.das_alloc.type      = _IO_DAS;
   io_msg.das_alloc.request   = _DAS_ALLOC; 
   io_msg.das_alloc.fd        = fd; 
   io_msg.das_alloc.flags     = flags;
   io_msg.das_alloc.n_bufs    = nbufs;
   io_msg.das_alloc.n_bytes   = nbytes;

   if(Sendfd(fd, &io_msg, &io_msg, sizeof(io_msg), sizeof(io_msg)) == -1)
      return(NULL);
   if (io_msg.status != EOK)
   {
      errno = io_msg.status;
      return(NULL);
   }
   if((dptr = (das_buf_t *)calloc(nbufs + 1, sizeof(das_buf_t))) == NULL)
      return(NULL);

   for(cnt = 0; cnt < nbufs; cnt++)
      if((dptr[cnt].bufp = malloc(nbytes)) == NULL)
      {
         das_free(fd, dptr);
         return(NULL);
      }
   return(dptr);
}

int das_free(int fd, das_buf_t *dptr)
{
   int cnt = 0;
                     // Fill in message structure.
   io_msg.das_alloc.type      = _IO_DAS;
   io_msg.das_alloc.request   = _DAS_FREE; 
   io_msg.das_alloc.fd        = fd; 

   if(Sendfd(fd, &io_msg, &io_msg, sizeof(io_msg), sizeof(io_msg)) == -1)
      return(-1);
   if (io_msg.status != EOK)
   {
      errno = io_msg.status;
      return(-1);
   }
   while(dptr[cnt].bufp)
      free(dptr[cnt++].bufp);
   free(dptr);
   return(EOK);
}
