//*****************************************************************************
// Module das_open_msg.c contains the open and close messaging functions for 
// the 'das' resource manager driver.  
//*****************************************************************************
// The follwing is the list of functions in this module.
//
//    das_open
//    das_close
//*****************************************************************************
#include <sys/kernel.h>
#include <sys/psinfo.h>
#include <sys/seginfo.h>
#include <sys/name.h>
#include <sys/proxy.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <das.h>
#include "das_struct_msg.h"

union das_msg io_msg;
das_info das_query_info;

//*****************************************************************************
// das_open -  Send a open message to the 'das' resource manager, then attach
//             proxies to use for A/D and D/A conversions and then query the
//             driver for information about the hardware.
//*****************************************************************************
#pragma off (unreferenced)
int das_open(const char *path, int oflag, const char *cfg_file)
{
   int fd;
                     // Open driver.
   if ((fd = open(path, oflag)) == -1)
      return(-1);
   
                     // Fill in message structure for query.
   io_msg.das_query.type = _IO_DAS;
   io_msg.das_query.request = _DAS_QUERY; 
   io_msg.das_query.fd = fd; 

   if(Sendfd(fd, &io_msg, &io_msg, sizeof(io_msg), sizeof(io_msg)) == -1)
   {
      das_close(fd);
      return(-1);
   }
   memcpy(&das_query_info, &io_msg.das_query_reply.d_info, 
            sizeof(das_info));   
   return(fd);
}

//*****************************************************************************
// das_get_info - 
//*****************************************************************************
int das_get_info(int fd, das_info *info)
{
   memcpy(info, &das_query_info, sizeof(das_info));
   return(EOK);
}
#pragma on (unreferenced)

//*****************************************************************************
// das_close - Send a close message to the driver 
//*****************************************************************************
int das_close(int fd)
{                    // Close driver.
   return(close(fd));
}

