// Advantech PCL-818 adatper
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// das_ad.c module contains routines to:
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//    perform a A/D conv via sw        - das_ad_sync()
//    perform 'N' A/D conv via
//       interrupts or dma             - das_ad_enqueue()
//    remove an entry from the finish
//       queue                         - das_ad_dequeue()
//    perform a blocking read on a
//       A/D request                   - das_ad_read()
//    perform a non-blocking read on a
//       A/D request                   - das_ad_cread()
//    terminate conversions on a chnl  - das_ad_term()
//    set the scan sequence for conv   - das_ad_set_scan()
//    return the range/gain on a chnl  - das_ad_get_range()
//    set the range/gain on a chnl     - das_ad_set_range()
//    set the trigger source on a chnl - das_ad_set_trig()
//    return the trig source on a chnl - das_ad_get_trig()
//    set the conv rate on a chnl      - das_ad_set_sample()
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#include <pcl_818.h>
#include <das_struct.h>
#include <das_dma.h>
#include <das_extern.h>
#include <das_inline.h>
#include <errno.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/kernel.h>
#include <sys/seginfo.h>

struct das_rng das_ad_rng[NUM_AD_RANGES] = {
          {  -5000000,  5000000 },        //   -5V - 5V
          {  -2500000,  2500000 },        // -2.5V - 2.5V
          {  -1000000,  1000000 },        //   -1V - 1V
          {   -500000,   500000 },        // -0.5V - 0.5V
          {         0, 10000000 },        //    0V - 10V
          {         0,  5000000 },        //    0V - 5V
          {         0,  2000000 },        //    0V - 2V
          {         0,  1000000 },        //    0V - 1V
          { -10000000, 10000000 } };      //  -10V - 10V

struct   das_scan dscan;               // scan queue

// Function das_ad_sync performs a single analog to digital conversion
int das_ad_sync(int fd, int chnl, char *buf)
{
   int cnt = 0;
   
   if(das_chk_init_fd(fd) != EOK)
      return(-1);
                                             // check for a valid channel
   if(chnl > ((docb.info.chnl_cnfg == DAS_SINGLE_ENDED) ? MAX_SNGL_CHNL : MAX_DIF_CHNL))
   {
      errno = ECHRNG;
      return(-1);
   }
   if(docb.ad[0].r_hd)
   {
      errno = EBUSY;
      return(-1);
   }
   das_set_mux(SET_SNGL_CHNL, chnl);         // set mux to specified chnl
   
   outp(docb.info.port+START_ADC, 0);        // start a conversion
   while(inp(docb.info.port+ADC_STATUS) & EOC)// wait for end of conversion
      if(cnt++ == 5000)                      // the hardware is baked!
      {
         errno = EIO;
         return(-1);
      }
   *(das_t *)buf = inpw(docb.info.port);     // read the data
   return(EOK);
}

// Function das_ad_dma_sync polls the DMA controller for terminal count.
int das_ad_dma_sync()
{
                                          // poll the dma controller
   while(read_dma(0, docb.info.dma_chnl) != 0xFFFF)
      ;
   das_ct_stop(_das_fd, 2);
   das_ct_stop(_das_fd, 3);
   outp(docb.info.port + ADC_CTL, 0x0);// disable the adapter

      //  unlink from request queue and link to finish queue
   das_link(das_unlink(0, &docb.ad[0].r_hd, &docb.ad[0].r_tl),
             &docb.ad[0].f_hd, &docb.ad[0].f_tl);
   docb.ad[0].n_rent--;
   docb.ad[0].n_fent++;
   docb.ad[0].stat = COMPLETE;
   Trigger(docb.ad[0].f_tl->proxy);
   return(0);
}

// Function das_ad_enqueue enqueues an A/D request.
int das_ad_enqueue(int fd, int chnl, das_buf_t *buf, int nbytes, int mode, pid_t proxy)
{
   switch(mode & OPER_MASK)               // check for a valid mode
   {     
      case DAS_INT:
      case DAS_DMA:
         return(das_ad_request(fd, chnl, buf, nbytes, mode, proxy));

      case DAS_DMA_SYNC:
         if(das_ad_request(fd, chnl, buf, nbytes, mode, proxy) !=EOK)
            return(-1);
         return(das_ad_dma_sync());

      default:
         errno = EINVAL;
         return(-1);
   }
}

// Function das_ad_request() places a conversion request into the request
// queue associated with 'chnl'.
int das_ad_request(int fd, int chnl, das_buf_t *buf, int nbytes, int mode, pid_t proxy)
{
   struct das_queue *ptr;

   if(das_chk_init_fd(fd) != EOK)            // library initialized?
      return(-1);

   if(chnl != 0)                             // only take channel zero for
   {                                         // adapters with multiplexed A/D
      errno = ECHRNG;
      return(-1);
   }

   if(docb.ad[0].r_hd && (mode & DAS_DMA_SYNC)) // no DMA_SYNC while requests
   {                                            // are in the queue
      errno = EBUSY;
      return(-1);
   }

   if(docb.da[0].r_hd)                       // adapter doesn't support A/D
   {                                         // and D/A at the same time
      errno = EBUSY;
      return(-1);
   }

   // limit number of entries in the queues
   if((docb.ad[0].n_rent + docb.ad[0].n_fent) == MAX_QUEUE_ENT)
   {
      errno = EBUSY;
      return(-1);
   }

   // make sure sample rate isn't too fast for interrupts
   if((mode & DAS_INT) && (docb.ad[0].s_rate.div1 * docb.ad[0].s_rate.div2)
          < INT_TICKS )
   {
      errno = EINVAL;
      return(-1);
   }

   // check for a valid number of conversions
   if(!nbytes || nbytes > MAX_CONV)
   {
      errno = EINVAL;
      return(-1);
   }

   // check for a valid mode
   if(((mode & MODE_MASK) != DAS_SNGL_CYCLE) && ((mode & MODE_MASK) != 
         DAS_RECYCLE) && ((mode & MODE_MASK) != DAS_NOTIFY))
   {
      errno = EINVAL;
      return(-1);
   }

   if(docb.ad[chnl].stat == DAS_RECYCLE)     // no sense queueing any more if
   {                                         // a recycle is in the queue
      errno = EBUSY;
      return(-1);
   }

   if(buf->bufp == NULL)                     // valid buffer?
   {
      errno = EINVAL;
      return(-1);
   }

   // create a request entry and fill it in
   if((ptr = das_create_entry()) == (struct das_queue *)NULL)
      return(-1);
   ptr->mode = mode & 0xFF;                  // mode bits 0 - 4
   ptr->rtype = mode & 0xF00;                // request type bits 8 - 11
   ptr->proxy = proxy;                       // proxy to trigger on completion 
   ptr->bytes = ptr->cnt = nbytes;           // number of bytes requested
   ptr->dma = buf;                           // structure from das_alloc
   ptr->cp = (das_t *)buf->bufp;             // pointer to buffer
   docb.ad[chnl].n_rent++;                   // increment num request entries
   docb.ad[chnl].stat = mode;                // set status to mode

   // link request to tail of request queue, if queue is emptry initiate
   // the request
   if(das_link(ptr, &docb.ad[chnl].r_hd, &docb.ad[chnl].r_tl))
      das_ad_cmd(chnl);
   return(EOK);
}

// Function das_ad_read performs a blocking read on a A/D request.  A check
// is made to make sure they don't block if they haven't initiated any
// conversions.  The proxy for the request is determined and the Receive
// function is called.  If DAS_REUSE is specified the request is reenqueued.
// The acquired data is copied into the specified buffer and the number of
// bytes is returned.
// Note this function will return EINVAL in a proxy hasn't been specified
// in the das_ad_enqueue function.  This function should not be called by
// servers.
int das_ad_read(int fd, char *buf, int nbytes, int chnl, int reuse)
{
   int psw, rtype;
   pid_t proxy;
   
   if(das_chk_init_fd(fd) != EOK)            // library initialized?
      return(-1);

   if(chnl != 0)                             // only take channel zero for
   {                                         // adapters with multiplexed A/D
      errno = ECHRNG;
      return(-1);
   }

   if(buf == NULL)                           // valid pointer?
   {
      errno = EINVAL;
      return(-1);
   }

   // Make sure they don't block if they haven't initiated any conversions
   psw = disable();
   if(docb.ad[chnl].f_hd == NULL && docb.ad[chnl].r_hd == NULL)
   {
      errno = EDEADLK;
      restore(psw);
      return(-1);
   }

   // proxies aren't triggered when we are in recycle mode
   if(docb.ad[chnl].stat == DAS_RECYCLE && docb.ad[chnl].f_hd == NULL)
   {
      errno = EDEADLK;
      restore(psw);
      return(-1);
   }
   restore(psw);
   if(!docb.ad[chnl].f_hd)                   // get the correct proxy to receive on
      proxy = docb.ad[chnl].r_hd->proxy;
   else
      proxy = docb.ad[chnl].f_hd->proxy;

   if(!proxy)                                // no proxy to block on
   {
      errno = EINVAL;
      return(-1);
   }

   while(!docb.ad[0].f_hd)                   // wait for an entry in the queue
   {
      if(Receive(proxy, 0, 0 ) == -1)        // when an entry is put in the
         return(-1);                         // queue we will always get
   }                                         // get a proxy to unblock us.
   
   // If they ask for more 'bytes' than was in the actual request, give them
   // the number of bytes in the request.  Otherwise give them what they
   // asked for.
   if(nbytes > docb.ad[chnl].f_hd->bytes)
      nbytes = docb.ad[chnl].f_hd->bytes;

   // copy the acquired data into the buffer passed in.
   das_memcpy16(buf, docb.ad[chnl].f_hd->dma->bufp, nbytes);

   // Make sure a D/A request isn't in progress they want to reuse the buffer.
   if(reuse && docb.da[0].r_hd)
   {
      das_unlink(1, &docb.ad[0].f_hd, &docb.ad[0].f_tl);
      docb.ad[0].n_fent--;
      errno = EBUSY;
      return(-1);
   }

   if(reuse)                              // re-enqueue the request?
   {
      rtype = docb.ad[0].f_hd->rtype;
      docb.ad[0].f_hd->cnt = docb.ad[0].f_hd->bytes;
      docb.ad[0].f_hd->cp = (das_t *)docb.ad[0].f_hd->dma->bufp;
      docb.ad[chnl].n_rent++; 
      docb.ad[chnl].stat = docb.ad[0].f_hd->mode;

      //  unlink from finish queue and link to request queue
      if(das_link(das_unlink(0, &docb.ad[0].f_hd, &docb.ad[0].f_tl),
               &docb.ad[0].r_hd, &docb.ad[0].r_tl))
         das_ad_cmd(chnl);
      if(rtype == DAS_DMA_SYNC)
         das_ad_dma_sync();
   }
   else              // unlink request from finish queue and free its memory
      das_unlink(1, &docb.ad[chnl].f_hd, &docb.ad[chnl].f_tl);
   docb.ad[0].n_fent--;             // decrement num entries in finish queue
   return(nbytes);                  // return the number of conversions
}

// Function das_ad_cread() performs a non-blocking read on a A/D request.
// The 'chnl' finish queue is checked for an entry.  If no entries exist in
// the queue, a 0 is returned.  Otherwise the entry it is unlinked from
// it and the number of conversions is returned.  If DAS_REUSE is specified
// the request is re-enqueued.
int das_ad_cread(int fd, char *buf, int nbytes, int chnl, int reuse)
{
   int rtype;
   
   if(das_chk_init_fd(fd) != EOK)            // library initialized?
      return(-1);

   if(chnl != 0)                             // only take channel zero for
   {                                         // adapters with multiplexed A/D
      errno = ECHRNG;
      return(-1);
   }

   if(buf == NULL)                           // valid buffer?
   {
      errno = EINVAL;
      return(-1);
   }

   if(!docb.ad[chnl].f_hd)                   // anything in the finish queue?
      return(0);

   // If they ask for more 'bytes' than was in the actual request, give them
   // the number of bytes in the request.  Otherwise give them what they
   // asked for.
   if(nbytes > docb.ad[chnl].f_hd->bytes)
      nbytes = docb.ad[0].f_hd->bytes;

   // copy the acquired data into the buffer passed in.
   das_memcpy16(buf, docb.ad[0].f_hd->dma->bufp, nbytes);

   // Make sure a D/A request isn't in progress they want to reuse the buffer.
   if(reuse && docb.da[0].r_hd)
   {
      das_unlink(1, &docb.da[0].f_hd, &docb.da[0].f_tl);
      docb.da[0].n_fent--;
      errno = EBUSY;
      return(-1);
   }
   if(reuse)                              // re-enqueue the request?
   {
      rtype = docb.ad[0].f_hd->rtype;
      docb.ad[0].f_hd->cnt = docb.ad[0].f_hd->bytes;
      docb.ad[0].f_hd->cp = (das_t *)docb.ad[0].f_hd->dma->bufp;
      docb.ad[chnl].n_rent++; 
      docb.ad[chnl].stat = docb.ad[0].f_hd->mode;

      //  unlink from finish queue and link to request queue
      if(das_link(das_unlink(0, &docb.ad[0].f_hd, &docb.ad[0].f_tl),
               &docb.ad[0].r_hd, &docb.ad[0].r_tl))
         das_ad_cmd(chnl);
      if(rtype == DAS_DMA_SYNC)
         das_ad_dma_sync();
   }
   else              // unlink request from finish queue and free its memory
      das_unlink(1, &docb.ad[chnl].f_hd, &docb.ad[chnl].f_tl);
   docb.ad[0].n_fent--;
   return(nbytes);                              // return the number of conversions
}

// Function das_ad_dequeue removes an entry from the finish queue and returns
// a pointer to it.  If there are no entries in the finish queue it returns
// with an error.
das_buf_t * das_ad_dequeue(int fd, int chnl)
{
   das_buf_t *tmp;

   if(das_chk_init_fd(fd) != EOK)            // library initialized?
      return(NULL);

   if(chnl != 0)                             // only take channel zero for
   {                                         // adapters with multiplexed A/D
      errno = ECHRNG;
      return(NULL);
   }

   if(!docb.ad[0].f_hd)                      // entries in finish queue?
   {
      errno = EACCES;
      return(NULL);
   }
   tmp = docb.ad[0].f_hd->dma;               // pointer to das_buf_t structure
      // unlink entry from finish queue and free the memory
   das_unlink(1, &docb.ad[chnl].f_hd, &docb.ad[chnl].f_tl);
   docb.ad[0].n_fent--;             // decrement num entries in finish queue
   return(tmp);
}

// Function das_ad_term() stops conversions on 'chnl'.  The entries in the
// 'ready' and 'finish' queues are removed.
int das_ad_term(int fd, int chnl)
{
   struct das_queue *r_hd, *f_hd, *r_tl, *f_tl;
   int psw;
   
   if(das_chk_init_fd(fd) != EOK)            // library initialized?
      return(-1);

   if(chnl != 0)                             // only take channel zero for
   {                                         // adapters with multiplexed A/D
      errno = ECHRNG;
      return(-1);
   }
   das_ct_stop(_das_fd, 2);                  // stop counters/timers
   das_ct_stop(_das_fd, 3);

   psw = disable();
   r_hd = docb.ad[chnl].r_hd;
   f_hd = docb.ad[chnl].f_hd;
   r_tl = docb.ad[chnl].r_tl;
   f_tl = docb.ad[chnl].f_tl;
   docb.ad[chnl].r_hd = docb.ad[chnl].r_tl = NULL;
   docb.ad[chnl].f_hd = docb.ad[chnl].f_tl = NULL;
   restore(psw);

// terminate the current interrupt or dma conversions
   if(r_hd)
   {
      docb.ad[chnl].stat = COMPLETE;
      outp(docb.info.port+ADC_CTL, 0);       // disable the pcl818 adapter
      outp(docb.info.port+PIT_ENABLE, docb.cntrs.pit_stat & 2);
      outp(docb.info.port + CLEAR_IRQ, 0x00);
      if(r_hd->rtype == DAS_DMA)
         clear_dma(0, docb.info.dma_chnl);
   }
   docb.ad[chnl].n_fent = 0;
   docb.ad[chnl].n_rent = 0;
   
   // clean up the queues
   while(r_hd)
      das_unlink(1, &r_hd, &r_tl);
   while(f_hd)
   {
      Creceive(f_hd->proxy, 0, 0);
      das_unlink(1, &f_hd, &f_tl);
   }
   return(EOK);
}

// Function das_set_mux() programs the gain/range queue ram and scan sequence.
// The PCL-818 must be set to DAS_REMOTE mode for the gain/range to be set.
int das_set_mux(int mode, int chnl)
{
   int cnt;

   if(mode == SET_SNGL_CHNL)
   {
      if(docb.info.range_ctrl == DAS_REMOTE)
      {                                            // Must be < 8 in diff mode
         outp(docb.info.port + ADC_MUX_CTL, chnl);
         outp(docb.info.port + RANGE_CTL, docb.ad[chnl].rng);
      }
      outp(docb.info.port + ADC_MUX_CTL, (chnl << 4) | chnl);
      return(EOK);
   }
   if(docb.info.range_ctrl == DAS_REMOTE)
      for(cnt=0;cnt<=dscan.last; cnt++)
      {                                            // Must be < 8 in diff mode
         outp(docb.info.port + ADC_MUX_CTL, dscan.scan[cnt]);
         outp(docb.info.port + RANGE_CTL, docb.ad[cnt].rng);
      }
   return(EOK);
}

// Function das_ad_set_scan() sets the scan queue.  The scan sequence is checked
// for validity and the hardware range/gain queuing RAM is programmed.
// The PCL-818 requires that the sequence must is consecutive. ie.
// 0, 1, 2, 3, 0, 1, 2, 3.
int das_ad_set_scan(int fd, char *scanlist, int n_entries)
{
   int cnt, chnl;
   
   if(das_chk_init_fd(fd) != EOK)         // library initialized?
      return(-1);

   if(docb.ad[0].r_hd)                    // don't allow scan sequence to
   {                                      // change until queue is empty
      errno = EBUSY;
      return(-1);
   }
                                          // check for valid number of entries
   if(n_entries <= 0 || n_entries > QUEUE_SIZE)
   {
      errno = EINVAL;
      return(-1);
   }
   chnl = (docb.info.chnl_cnfg == DAS_SINGLE_ENDED) ? MAX_SNGL_CHNL : MAX_DIF_CHNL;
   for(cnt=0; cnt<n_entries-1; cnt++)
      if(((scanlist[cnt+1] - scanlist[cnt] != 1) && scanlist[cnt] != chnl) ||
          scanlist[cnt] > chnl)
      {
         errno = ECHRNG;
         return(-1);
      }
   memcpy(&dscan.scan[0], scanlist, n_entries);
   dscan.last = n_entries-1;
   das_set_mux(SET_ALL_CHNLS, 0);
   return(EOK);
}

// Function das_ad_get_range() returns the range/gain on 'chnl'.
int das_ad_get_range(int fd, int chnl, long *rmin, long *rmax)
{
   if(das_chk_init_fd(fd) != EOK)
      return(-1);
                                          // check for a valid channel number
   if(chnl > ((docb.info.chnl_cnfg == DAS_SINGLE_ENDED) ? MAX_SNGL_CHNL : MAX_DIF_CHNL))
   {
      errno = ECHRNG;
      return(-1);
   }

   *rmin = das_ad_rng[docb.ad[chnl].rng].min;
   *rmax = das_ad_rng[docb.ad[chnl].rng].max;
   return(EOK);
}

// Function das_ad_set_range() sets the range/gain on 'chnl'.  Returns zero on
// success -1 on error.
int das_ad_set_range(int fd, int chnl, long rmin, long rmax)
{
   int cnt;
   
   if(das_chk_init_fd(fd) != EOK)            // library initialized?
      return(-1);

   if(docb.info.range_ctrl == DAS_LOCAL)     // can't set range if in DAS_LOCAL mode
   {
      errno = EINVAL;
      return(-1);
   }
   if(docb.ad[0].r_hd)                    // don't allow range change
   {                                      // until queue is empty
      errno = EBUSY;
      return(-1);
   }
                                          // check for a valid channel number
   if(chnl > ((docb.info.chnl_cnfg == DAS_SINGLE_ENDED) ? MAX_SNGL_CHNL : MAX_DIF_CHNL))
   {
      errno = ECHRNG;
      return(-1);
   }

   for(cnt=0;cnt<NUM_AD_RANGES; cnt++)
      if(das_ad_rng[cnt].min == rmin && das_ad_rng[cnt].max == rmax)
         break;
   
   if(cnt == NUM_AD_RANGES)
   {
      errno = EINVAL;
      return(-1);
   }
   docb.ad[chnl].rng = cnt;
   das_set_mux(SET_ALL_CHNLS, 0);
   return(EOK);
}

// NOTE: TRIG0 and GATE0 Selection
//       JP1 must be set to C where TRIG0 and DIO are shorted together
//       JP2 must be set to C where GATE0 and DI2 are shorted together

// ADC control register bits for triggering
// ST0   ST1
//  0    X     software trigger
//  1    0     external trigger
//  1    1     pacer trigger

// Timer/Counter Enable register bits for gating
// TC0
//  0          pacer is enabled
//  1          disable pacer until TRIG0 is high

// Pacer trigger can be gated by setting TC0 high.  External trigger can
// not be gated.

// Function das_ad_set_trig() sets the trigger source on 'chnl'.
int das_ad_set_trig(int fd, int chnl, int trig_src)
{
   int trig;
   
   if(das_chk_init_fd(fd) != EOK)            // library initialized?
      return(-1);

   if(docb.ad[0].r_hd)                       // don't allow a trigger change
   {                                         // until queue is empty
      errno = EBUSY;
      return(-1);
   }

   if(chnl != 0)                 // Just allow channel zero trigger to be set
   {
      errno = ECHRNG;
      return(-1);
   }

   switch(trig_src)                       // valid trigger source?
   {
      case DAS_SW_TRIG:
         trig = _SW_TRIG;
         break;
      case DAS_PC_TRIG:
         trig = _PC_TRIG;
         break;
      case DAS_PC_TRIG | DAS_ENB_GATE:
         trig = _PC_TRIG | _ENB_GATE;
         break;
      case DAS_EX_TRIG:
         trig = _EX_TRIG;
         break;
      default:
         errno = EINVAL;
         return(-1);
   }
   docb.ad[chnl].trig_src = trig;
   return(EOK);
}

// Function das_ad_get_trig() returns the trigger source on 'chnl'.
int das_ad_get_trig(int fd, int chnl, int *trig_src)
{
   if(das_chk_init_fd(fd) != EOK)            // library initialized?
      return(-1);

   if(chnl > ((docb.info.chnl_cnfg == DAS_SINGLE_ENDED) ? MAX_SNGL_CHNL : MAX_DIF_CHNL))
   {
      errno = ECHRNG;
      return(-1);
   }
      // convert trigger back to manifests
   if(docb.ad[chnl].trig_src & _PC_TRIG)
      *trig_src = (docb.ad[chnl].trig_src & 0xF8) | DAS_PC_TRIG;
   else
      *trig_src = docb.ad[chnl].trig_src;
   return(EOK);
}

// Function das_ad_set_sample() sets the conversion rate on 'chnl'.
int das_ad_set_sample(int fd, int chnl, unsigned long ticks)
{
   if(das_chk_init_fd(fd) != EOK)            // library initialized?
      return(-1);

   if(chnl > ((docb.info.chnl_cnfg == DAS_SINGLE_ENDED) ? MAX_SNGL_CHNL : MAX_DIF_CHNL))
   {
      errno = ECHRNG;
      return(-1);
   }

   if(ticks < MIN_TICKS || ticks > MAX_TICKS)   // valid number of ticks?
   {
      errno = EINVAL;
      return(-1);
   }
      
   // convert ticks parameter into two divisors that can be sent to counters
   das_convert_rate(ticks, &docb.ad[chnl].s_rate);
   docb.cntrs.ctr[3-1].ld = docb.ad[chnl].s_rate.div2;
   docb.cntrs.ctr[2-1].ld = docb.ad[chnl].s_rate.div1;
   if(docb.ad[chnl].r_hd && (docb.ad[chnl].trig_src & DAS_PC_TRIG))
   {
      das_ct_write(_das_fd, 2, &docb.cntrs.ctr[2-1]);
      das_ct_write(_das_fd, 3, &docb.cntrs.ctr[3-1]);
   }
   return(EOK);
}

// Function das_convert_rate() converts a sample rate in ticks into two divisor
// values that can be written to the counters.
int das_convert_rate(unsigned long ticks, struct das_rate *sptr)
{
   sptr->div1 = 0;
   sptr->div2 = 1;
   
   do
   {
      sptr->div2*=2;
      sptr->div1 = ticks / sptr->div2;
   }while(sptr->div1 > 0xFFFF);
   return(EOK);
}

// Function das_ad_rqueue returns the number of requests in the ready queue
// for a 'chnl'.
int das_ad_rqueue(int fd, int chnl)
{
   if(das_chk_init_fd(fd) != EOK)               // libary initialized?
      return(-1);

   if(chnl > ((docb.info.chnl_cnfg == DAS_SINGLE_ENDED) ? MAX_SNGL_CHNL : MAX_DIF_CHNL))
   {
      errno = ECHRNG;
      return(-1);
   }
   return(docb.ad[0].n_rent);
}

// Function das_ad_fqueue returns the number of requests in the ready queue
// for a 'chnl'.
int das_ad_fqueue(int fd, int chnl)
{
   if(das_chk_init_fd(fd) != EOK)               // libary initialized?
      return(-1);

   if(chnl > ((docb.info.chnl_cnfg == DAS_SINGLE_ENDED) ? MAX_SNGL_CHNL : MAX_DIF_CHNL))
   {
      errno = ECHRNG;
      return(-1);
   }
   return(docb.ad[0].n_fent);
}
