// Advantech PCL-818 adapter
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// das_da.c module contains routines to:
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//    perform a D/A conv via sw        - das_da_sync()
//    perform 'N' A/D conv via
//       interrupts or dma             - das_da_enqueue()
//    terminate conversions on a chnl  - das_da_term()
//    return the range/gain on a chnl  - das_da_get_range()
//    set the range/gain on a chnl     - das_da_set_range()
//    set the trigger source on a chnl - das_da_set_trig()
//    return the trig source on a chnl - das_da_get_trig()
//    set the conv rate on a chnl      - das_da_set_sample()
//    get the status on an conv        - das_da_cwait()
//    wait for a D/A request to finish - das_da_wait()
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#include <errno.h>
#include <pcl_818.h>
#include <stdlib.h>
#include <conio.h>
#include <das_extern.h>
#include <das_struct.h>
#include <das_dma.h>
   
// Function das_da_sync performs a single digital to analog conversion
int das_da_sync(int fd, int chnl, char *buf)
{
   if(das_chk_init_fd(fd) != EOK)
      return(-1);

   if(chnl > MAX_DA_CHNL)                    // check for valid channel here
   {
      errno = ECHRNG;
      return(-1);
   }
   if(buf == NULL)
   {
      errno = EINVAL;
      return(-1);
   }
   if(docb.da[0].r_hd)
   {
      errno = EBUSY;
      return(-1);
   }
   outpw(docb.info.port+docb.da[chnl].addr, *(das_t *)buf);
   return(EOK);
}

#pragma off (unreferenced)
int das_da_enqueue(int fd, int chnl, das_buf_t *buf, int nbytes, int mode, pid_t proxy)
{
// pcl818 does not support digital to analog conversions via dma or interrupts.
   errno = ENOTSUP;
   return(-1);
}

// Function das_da_term() stops D/A conversions on 'chnl'.
int das_da_term(int fd, int chnl)
{
   errno = ENOTSUP;
   return(-1);
}

// Function das_da_stat() returns the status of a D/A 'chnl.
int das_da_cwait(int fd, int chnl, int reuse)
{
   errno = ENOTSUP;
   return(-1);
}

// Function das_da_wait() waits for a D/A request to complete on 'chnl'.
int das_da_wait(int fd, int chnl, int reuse)
{
   errno = ENOTSUP;
   return(-1);
}

// Function das_da_get_range() returns the output range on D/A 'chnl'.
int das_da_get_range(int fd, int chnl, long *rmin, long *rmax)
{
// pcl818 does not support programable range
   errno = ENOTSUP;
   return(-1);
}

// Function das_da_set_range() sets the range on D/A 'chnl'.
int das_da_set_range(int fd, int chnl, long rmin, long rmax)
{
// pcl818 does not support programable range
   errno = ENOTSUP;
   return(-1);
}

// Function das_da_get_trig() returns the trigger source on D/A 'chnl'.
int das_da_get_trig(int fd, int chnl, int *trig_src)
{
// PCL-818 doesn't support this one.
   errno = ENOTSUP;
   return(-1);
}

// Function das_da_set_trig() sets the trigger source on D/A 'chnl'.
int das_da_set_trig(int fd, int chnl, int trig_src)
{
// PCL-818 doesn't support this one.
   errno = ENOTSUP;
   return(-1);
}

// Function das_da_set_sample() sets the conversion rate and D/A 'chnl'.
int das_da_set_sample(int fd, int chnl, unsigned long ticks)
{
// PCL-818 doesn't support this one either.
   errno = ENOTSUP;
   return(-1);
}
#pragma on (unreferenced)

// Function das_chk_init_fd() checks for a valid file descriptor and that the
// library has been initialized.
int das_chk_init_fd(int fd)
{
   if(!init_flag)
   {
      errno = EBADF;
      return(-1);
   }
   if(fd != _das_fd)
   {
      errno = EBADF;
      return(-1);
   }
   return(EOK);
}

#pragma off (unreferenced)
// Function das_da_rqueue returns the number of requests in the ready queue
// for a 'chnl'.
int das_da_rqueue(int fd, int chnl)
{
   errno = ENOTSUP;
   return(-1);
}

// Function das_da_fqueue returns the number of requests in the ready queue
// for a 'chnl'.
int das_da_fqueue(int fd, int chnl)
{
   errno = ENOTSUP;
   return(-1);
}
#pragma on (unreferenced)
