// Advantech PCL-818 adapter
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// das_int.c module contains routines to:
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//    interrupt handler                - das_handler()
//    program adapter for ad conver.   - das_ad_cmd()
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// If the memory model isn't small, compile with the -zu option
// das_int.c module contains the interrupt handler for the adapter

#include <conio.h>
#include <stdio.h>
#include <errno.h>
#include <pcl_818.h>
#include <das_extern.h>
#include <das_dma.h>
#include <das_struct.h>
#include <das.h>
#include <das_inline.h>

#ifdef _DAS_MAN
volatile das_t das_res_buf[16];
extern int das_res_int(void);
#endif

#pragma off (check_stack)
// Function das_handler() processes interrupts from the adapter.  Possible
// interrupt sources are:  DMA terminal count
//                         End Conversion
pid_t far das_handler()
{
   register struct das_chnl *p = &docb.ad[0];
   
   if(docb.ad[0].r_hd == NULL && docb.da[0].r_hd == NULL)
      return(0);
   if(!(inp(docb.info.port + ADC_CTL) & 0x04))  // DMA complete interrupt?
   {
      if((p->r_hd->mode & DAS_RECYCLE) && !p->r_hd->cnt)
      {
         p->r_hd->cp = (das_t *)p->r_hd->dma->bufp;
         p->r_hd->cnt = p->r_hd->bytes;
      }
      *p->r_hd->cp++ = inpw(docb.info.port + ADC_DATA);
      p->r_hd->cnt -= sizeof(das_t);

      // if we haven't done all the conversions or we're in DAS_RECYCLE mode
      // clear the interrupt and return
      if(p->r_hd->cnt || (p->r_hd->mode & DAS_RECYCLE))
      {
         outp(docb.info.port+CLEAR_IRQ,0);         // clear the das interrupt
         return(0);
      }
   }
#ifdef _DAS_MAN
// This section of code is required for the Acquisition Manager.  We use the
// DMA terminal count as a time base.  This time base is scaled to produce
// the conversion rate specified by the das_ad_set_sample and das_da_set_sample
// functions.  When the DMA transfer completes the DMAC is re-programmed and
// the das_res_int function is called.  The das_res_int function increments a
// wall clock.  The wall clock is then compared to the first entry in the
// tick queue.  Equal values indicate that a conversion should be performed.
// A proxy is triggered to signify this event.  This event is then processed
// by the manager at process time.

   if(p->r_hd->mode & DAS_NOTIFY)         // used by resource manager
   {
         // copy data from active sample buffer to buffer for manager
      das_memcpy16(das_res_buf, p->r_hd->dma->bufp, p->r_hd->bytes);
      outp(docb.info.port+CLEAR_IRQ,0);         // clear the das interrupt
      das_ad_cmd(0);
      if (das_res_int())
         return(docb.ad[0].r_hd->proxy);
      return(0);
   }
#endif

   das_ct_stop(_das_fd, 2);
   das_ct_stop(_das_fd, 3);

   //  unlink from request queue and link to finish queue
   das_link(das_unlink(0, &p->r_hd, &p->r_tl), &p->f_hd, &p->f_tl);
   p->n_rent--;            // decrement num entries in request queue
   p->n_fent++;            // increment num entries in finish queue
   if(p->r_hd)                            // more requests to process?
      das_ad_cmd(0);                      // process next entry
   else                                   // no more requests
      docb.ad[0].stat = COMPLETE;
   return(docb.ad[0].f_tl->proxy);        // trigger a proxy indicating a
                                          // request has completed
}
#pragma on (check_stack)

#pragma off (check_stack)
// Function das_ad_cmd() programs the adapter using the information stored
// in the queue request entry
void das_ad_cmd(int chnl)
{
   int dflags, psw;
   
   outp(docb.info.port + ADC_CTL, 0x0);      // disable the adapter

   das_ct_stop(_das_fd, 2);                  // stop the counters
   das_ct_stop(_das_fd, 3);

   if((docb.ad[chnl].trig_src == _EX_TRIG) ||
                           (docb.ad[chnl].trig_src == (_PC_TRIG | _ENB_GATE)))
      docb.cntrs.pit_stat |= DIS_PACER;      // external trig or pc trig with gate
   else
      docb.cntrs.pit_stat &= ENB_PACER;      // just pacer

   if(docb.ad[chnl].r_hd->rtype & DAS_DMA || 
               docb.ad[chnl].r_hd->rtype & DAS_DMA_SYNC)
   {
      if(docb.ad[chnl].r_hd->mode & DAS_RECYCLE)
         dflags = DMA_AUTO | DMA_WAIT;
      else
         dflags = DMA_WAIT;
      setup_dma(dflags, &docb.ad[chnl].r_hd->dma->dma,
                  docb.ad[chnl].r_hd->bytes, docb.info.dma_chnl);

      dflags = EN_DMA;

      // if it is in RECYCLE or DMA_SYNC mode don't enable the interrupts
      if(!(docb.ad[chnl].r_hd->mode & DAS_RECYCLE) && 
                        docb.ad[chnl].r_hd->rtype != DAS_DMA_SYNC)
         dflags |= EN_INT;
   }
   else
      dflags = EN_INT;                          // interrupt request
   outp(docb.info.port + PIT_ENABLE, docb.cntrs.pit_stat);
   outp(docb.info.port + CLEAR_IRQ, 0x00);      // clear pending interrupts
                                                // set the multiplexer
   outp(docb.info.port + ADC_MUX_CTL, (dscan.scan[dscan.last] << 4)
            | dscan.scan[0]);
   outp(docb.info.port + ADC_CTL, dflags | (docb.info.irq << 4) |
                                             (docb.ad[chnl].trig_src & 0x3));
   if(docb.ad[chnl].trig_src & _PC_TRIG)
   {
      das_ct_mode(_das_fd, 2, &docb.cntrs.ctr[1].md, NULL); // cntr 2
      das_ct_write(_das_fd, 2, &docb.cntrs.ctr[1]);
      das_ct_mode(_das_fd, 3, &docb.cntrs.ctr[2].md, NULL); // cntr 3
      psw = disable();
      das_ct_write(_das_fd, 3, &docb.cntrs.ctr[2]);
   }
   // start dma
   if((docb.ad[chnl].r_hd->rtype & DAS_DMA) ||
                                 (docb.ad[chnl].r_hd->rtype & DAS_DMA_SYNC))
      kick_dma(0, docb.info.dma_chnl);
   restore(psw);
}
#pragma on (check_stack)
