//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// das_queue.c module contains routines to:
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//    perform an A/D conv via sw       - das_ad_poll()
//    allocate a queue entry           - das_create_entry()
//    unlink an entry from the queue   - das_unlink()
//    link an entry to a queue         - das_link()
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#include <das_inline.h>
#include <stdlib.h>
#include <errno.h>
#include <i86.h>
#include <pcl_818.h>
#include <das_struct.h>
#include <das_extern.h>

#pragma off (check_stack)
// Function das_create_entry() allocates a entry for the queue and returns a pointer
// to it
struct das_queue * das_create_entry(void)
{
   struct das_queue *ptr;

   if((ptr = (struct das_queue *)calloc(1, sizeof(struct das_queue))) == NULL)
      errno = ENOMEM;
   return(ptr);
}

// Function das_unlink() unlinks the entry from the queue and free it's
// memory if 'op' is true.  Note: do not call this routine from an interrupt
// handler with 'op' set true.
struct das_queue * das_unlink(int op, struct das_queue **hptr, struct das_queue **tptr)
{
   struct   das_queue   *tmp;
   unsigned psw;
   
   psw = disable();
   tmp = *hptr;
   *hptr =(*hptr)->nptr;
   if(!*hptr)
      *tptr = NULL;
   restore(psw);
   if(op)                           // free the memory
      free(tmp);
   return(tmp);
}

// Function das_link() links an entry to the tail of the queue.  It returns
// true if it is the first entry in the queue.  This return value is used
// so we can initiate conversions when the adapter is idle.  Otherwise we
// rely on the interrupt handler to process the next entry in the queue.
int das_link(struct das_queue *ptr, struct das_queue **hptr, struct das_queue **tptr)
{
   unsigned psw;
   
   psw = disable();
   ptr->nptr = NULL;
   // If there is already entries in the queue put it at the tail and return
   if(*tptr)
   {
      (*tptr)->nptr = ptr;
      *tptr = ptr;
   }
   else              // Nothing in the queue, initiate conversions
   {
      *hptr = *tptr = ptr;
      restore(psw);
      return(1);
   }
   restore(psw);
   return(EOK);
}
#pragma on (check_stack)
